"""
Shapefile读取器模块 - 提供从Shapefile文件读取地理数据的功能
"""
from typing import Dict, List, Any, Tuple, Optional
import os
import warnings

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..core.graph import LocationGraph
from ..core.exceptions import DataFormatError, IOError


class ShapefileReader:
    """
    Shapefile读取器，用于从Shapefile文件读取地理数据
    
    注意：此类依赖于geopandas库，需要先安装：
    pip install geopandas
    """
    
    @staticmethod
    def read_file(file_path: str) -> Any:
        """
        读取Shapefile文件
        
        参数:
            file_path: Shapefile文件路径
            
        返回:
            GeoDataFrame对象
            
        异常:
            IOError: 如果文件读取失败
            ImportError: 如果未安装geopandas
        """
        try:
            import geopandas as gpd
        except ImportError:
            raise ImportError("未安装geopandas库，请先安装: pip install geopandas")
        
        try:
            # 读取Shapefile
            gdf = gpd.read_file(file_path)
            return gdf
        
        except Exception as e:
            raise IOError(f"读取Shapefile失败: {str(e)}")
    
    @staticmethod
    def parse_point(geometry, properties: Dict[str, Any]) -> POI:
        """
        解析点几何体为POI对象
        
        参数:
            geometry: 点几何体
            properties: 属性字典
            
        返回:
            POI对象
        """
        try:
            import shapely
        except ImportError:
            raise ImportError("未安装shapely库，请先安装: pip install shapely")
        
        # 获取坐标
        lon, lat = geometry.x, geometry.y
        
        # 创建坐标对象
        coord = Coordinate(lat, lon)
        
        # 创建POI对象
        poi_data = {
            'coordinate': coord,
            'poi_id': properties.get('id') or properties.get('poi_id'),
            'name': properties.get('name', ''),
            'poi_type': properties.get('type') or properties.get('poi_type', 'generic'),
            'properties': properties
        }
        
        # 如果没有提供ID，使用坐标格式作为ID
        if not poi_data['poi_id']:
            poi_data['poi_id'] = f"node_{coord.longitude:.6f}_{coord.latitude:.6f}"
        
        return POI(
            coordinate=coord,
            poi_id=poi_data['poi_id'],
            name=poi_data['name'],
            poi_type=poi_data['poi_type'],
            properties=poi_data['properties']
        )
    
    @staticmethod
    def parse_linestring(geometry, properties: Dict[str, Any]) -> Road:
        """
        解析线几何体为Road对象
        
        参数:
            geometry: 线几何体
            properties: 属性字典
            
        返回:
            Road对象
        """
        try:
            import shapely
        except ImportError:
            raise ImportError("未安装shapely库，请先安装: pip install shapely")
        
        # 获取坐标
        coords = []
        for point in geometry.coords:
            lon, lat = point[0], point[1]
            elevation = point[2] if len(point) > 2 else None
            coords.append(Coordinate(lat, lon, elevation))
        
        # 创建Road对象
        road_data = {
            'road_id': properties.get('id') or properties.get('road_id'),
            'name': properties.get('name', ''),
            'start_node_id': properties.get('start_node_id'),
            'end_node_id': properties.get('end_node_id'),
            'coordinates': coords,
            'road_type': properties.get('type') or properties.get('road_type', 'street'),
            'length': properties.get('length'),
            'speed_limit': properties.get('speed_limit', 50.0),
            'bidirectional': properties.get('bidirectional', True),
            'properties': properties
        }
        
        # 如果没有提供ID，使用坐标的哈希值作为ID
        if not road_data['road_id']:
            road_data['road_id'] = f"road_{hash(tuple(coords))}"
        
        # 如果没有提供节点ID，使用坐标格式作为节点ID
        if not road_data['start_node_id']:
            start_coord = coords[0]
            road_data['start_node_id'] = f"node_{start_coord.longitude:.6f}_{start_coord.latitude:.6f}"
        
        if not road_data['end_node_id']:
            end_coord = coords[-1]
            road_data['end_node_id'] = f"node_{end_coord.longitude:.6f}_{end_coord.latitude:.6f}"
        
        return Road(
            road_id=road_data['road_id'],
            name=road_data['name'],
            start_node_id=road_data['start_node_id'],
            end_node_id=road_data['end_node_id'],
            coordinates=road_data['coordinates'],
            road_type=road_data['road_type'],
            length=road_data['length'],
            speed_limit=road_data['speed_limit'],
            bidirectional=road_data['bidirectional'],
            properties=road_data['properties']
        )
    
    @classmethod
    def load_graph(cls, file_path: str) -> LocationGraph:
        """
        从Shapefile文件加载地理空间图
        
        参数:
            file_path: Shapefile文件路径
            
        返回:
            LocationGraph对象
            
        异常:
            IOError: 如果文件读取失败
            DataFormatError: 如果文件格式不正确
        """
        try:
            import shapely
        except ImportError:
            raise ImportError("未安装shapely库，请先安装: pip install shapely")
        
        # 读取Shapefile
        gdf = cls.read_file(file_path)
        
        # 创建图
        graph = LocationGraph()
        
        # 解析几何体
        for _, row in gdf.iterrows():
            geometry = row.geometry
            properties = {col: row[col] for col in gdf.columns if col != 'geometry'}
            
            try:
                # 解析点几何体
                if geometry.geom_type == 'Point':
                    poi = cls.parse_point(geometry, properties)
                    graph.add_node(poi)
                
                # 解析线几何体
                elif geometry.geom_type == 'LineString':
                    road = cls.parse_linestring(geometry, properties)
                    
                    # 检查起点和终点是否存在，如果不存在则创建
                    if not graph.has_node(road.start_node_id):
                        start_coord = road.get_start_coordinate()
                        start_poi = POI(
                            coordinate=start_coord,
                            poi_id=road.start_node_id,
                            name=f"Junction {road.start_node_id}",
                            poi_type="junction"
                        )
                        graph.add_node(start_poi)
                    
                    if not graph.has_node(road.end_node_id):
                        end_coord = road.get_end_coordinate()
                        end_poi = POI(
                            coordinate=end_coord,
                            poi_id=road.end_node_id,
                            name=f"Junction {road.end_node_id}",
                            poi_type="junction"
                        )
                        graph.add_node(end_poi)
                    
                    graph.add_edge(road)
                
                # 处理MultiLineString - 拆分为多个LineString
                elif geometry.geom_type == 'MultiLineString':
                    for line in geometry.geoms:
                        # 复制属性并添加原始ID后缀
                        multi_props = properties.copy()
                        if 'id' in multi_props:
                            multi_props['id'] = f"{multi_props['id']}_{hash(line)}"
                        
                        road = cls.parse_linestring(line, multi_props)
                        
                        # 检查起点和终点是否存在，如果不存在则创建
                        if not graph.has_node(road.start_node_id):
                            start_coord = road.get_start_coordinate()
                            start_poi = POI(
                                coordinate=start_coord,
                                poi_id=road.start_node_id,
                                name=f"Junction {road.start_node_id}",
                                poi_type="junction"
                            )
                            graph.add_node(start_poi)
                        
                        if not graph.has_node(road.end_node_id):
                            end_coord = road.get_end_coordinate()
                            end_poi = POI(
                                coordinate=end_coord,
                                poi_id=road.end_node_id,
                                name=f"Junction {road.end_node_id}",
                                poi_type="junction"
                            )
                            graph.add_node(end_poi)
                        
                        graph.add_edge(road)
                
                # 其他几何类型暂不支持
                else:
                    warnings.warn(f"不支持的几何类型: {geometry.geom_type}")
            
            except Exception as e:
                warnings.warn(f"解析几何体失败: {str(e)}")
        
        return graph
    
    @classmethod
    def load_pois(cls, file_path: str) -> List[POI]:
        """
        从Shapefile文件加载POI列表
        
        参数:
            file_path: Shapefile文件路径
            
        返回:
            POI对象列表
            
        异常:
            IOError: 如果文件读取失败
            DataFormatError: 如果文件格式不正确
        """
        try:
            import shapely
        except ImportError:
            raise ImportError("未安装shapely库，请先安装: pip install shapely")
        
        # 读取Shapefile
        gdf = cls.read_file(file_path)
        
        pois = []
        
        # 解析几何体
        for _, row in gdf.iterrows():
            geometry = row.geometry
            properties = {col: row[col] for col in gdf.columns if col != 'geometry'}
            
            try:
                # 解析点几何体
                if geometry.geom_type == 'Point':
                    poi = cls.parse_point(geometry, properties)
                    pois.append(poi)
                
                # 其他几何类型暂不支持
                else:
                    warnings.warn(f"不支持的几何类型: {geometry.geom_type}")
            
            except Exception as e:
                warnings.warn(f"解析几何体失败: {str(e)}")
        
        return pois
    
    @classmethod
    def load_roads(cls, file_path: str) -> List[Road]:
        """
        从Shapefile文件加载Road列表
        
        参数:
            file_path: Shapefile文件路径
            
        返回:
            Road对象列表
            
        异常:
            IOError: 如果文件读取失败
            DataFormatError: 如果文件格式不正确
        """
        try:
            import shapely
        except ImportError:
            raise ImportError("未安装shapely库，请先安装: pip install shapely")
        
        # 读取Shapefile
        gdf = cls.read_file(file_path)
        
        roads = []
        
        # 解析几何体
        for _, row in gdf.iterrows():
            geometry = row.geometry
            properties = {col: row[col] for col in gdf.columns if col != 'geometry'}
            
            try:
                # 解析线几何体
                if geometry.geom_type == 'LineString':
                    road = cls.parse_linestring(geometry, properties)
                    roads.append(road)
                
                # 其他几何类型暂不支持
                else:
                    warnings.warn(f"不支持的几何类型: {geometry.geom_type}")
            
            except Exception as e:
                warnings.warn(f"解析几何体失败: {str(e)}")
        
        return roads
    
    @staticmethod
    def save_graph(graph: LocationGraph, file_path: str) -> None:
        """
        将地理空间图保存为Shapefile文件
        
        参数:
            graph: LocationGraph对象
            file_path: 保存路径
            
        异常:
            IOError: 如果文件保存失败
            ImportError: 如果未安装geopandas
        """
        try:
            import geopandas as gpd
            from shapely.geometry import Point, LineString
        except ImportError:
            raise ImportError("未安装geopandas或shapely库，请先安装: pip install geopandas shapely")
        
        # 创建POI的GeoDataFrame
        poi_data = []
        for node in graph.nodes.values():
            geometry = Point(node.coordinate.longitude, node.coordinate.latitude)
            properties = {
                'id': node.poi_id,
                'name': node.name,
                'type': node.poi_type
            }
            
            # 添加其他属性
            for key, value in node.properties.items():
                properties[key] = value
            
            poi_data.append({**properties, 'geometry': geometry})
        
        # 创建Road的GeoDataFrame
        road_data = []
        for edge in graph.edges.values():
            # 转换坐标
            coords = [(coord.longitude, coord.latitude) for coord in edge.coordinates]
            geometry = LineString(coords)
            
            properties = {
                'id': edge.road_id,
                'name': edge.name,
                'type': edge.road_type,
                'start_node_id': edge.start_node_id,
                'end_node_id': edge.end_node_id,
                'length': edge.length,
                'speed_limit': edge.speed_limit,
                'bidirectional': edge.bidirectional
            }
            
            # 添加其他属性
            for key, value in edge.properties.items():
                properties[key] = value
            
            road_data.append({**properties, 'geometry': geometry})
        
        # 创建GeoDataFrame
        if poi_data and road_data:
            # 如果同时有POI和Road，分别保存
            poi_gdf = gpd.GeoDataFrame(poi_data)
            road_gdf = gpd.GeoDataFrame(road_data)
            
            # 获取文件名和目录
            file_dir = os.path.dirname(file_path)
            file_name = os.path.splitext(os.path.basename(file_path))[0]
            
            # 保存POI
            poi_path = os.path.join(file_dir, f"{file_name}_poi.shp")
            poi_gdf.to_file(poi_path)
            
            # 保存Road
            road_path = os.path.join(file_dir, f"{file_name}_road.shp")
            road_gdf.to_file(road_path)
        
        elif poi_data:
            # 只有POI
            poi_gdf = gpd.GeoDataFrame(poi_data)
            poi_gdf.to_file(file_path)
        
        elif road_data:
            # 只有Road
            road_gdf = gpd.GeoDataFrame(road_data)
            road_gdf.to_file(file_path)
        
        else:
            # 没有数据
            warnings.warn("没有数据可保存")
    
    @staticmethod
    def save_pois(pois: List[POI], file_path: str) -> None:
        """
        将POI列表保存为Shapefile文件
        
        参数:
            pois: POI对象列表
            file_path: 保存路径
            
        异常:
            IOError: 如果文件保存失败
            ImportError: 如果未安装geopandas
        """
        try:
            import geopandas as gpd
            from shapely.geometry import Point
        except ImportError:
            raise ImportError("未安装geopandas或shapely库，请先安装: pip install geopandas shapely")
        
        # 创建POI的GeoDataFrame
        poi_data = []
        for poi in pois:
            geometry = Point(poi.coordinate.longitude, poi.coordinate.latitude)
            properties = {
                'id': poi.poi_id,
                'name': poi.name,
                'type': poi.poi_type
            }
            
            # 添加其他属性
            for key, value in poi.properties.items():
                properties[key] = value
            
            poi_data.append({**properties, 'geometry': geometry})
        
        # 创建GeoDataFrame
        if poi_data:
            poi_gdf = gpd.GeoDataFrame(poi_data)
            poi_gdf.to_file(file_path)
        else:
            warnings.warn("没有POI数据可保存")
    
    @staticmethod
    def save_roads(roads: List[Road], file_path: str) -> None:
        """
        将Road列表保存为Shapefile文件
        
        参数:
            roads: Road对象列表
            file_path: 保存路径
            
        异常:
            IOError: 如果文件保存失败
            ImportError: 如果未安装geopandas
        """
        try:
            import geopandas as gpd
            from shapely.geometry import LineString
        except ImportError:
            raise ImportError("未安装geopandas或shapely库，请先安装: pip install geopandas shapely")
        
        # 创建Road的GeoDataFrame
        road_data = []
        for road in roads:
            # 转换坐标
            coords = [(coord.longitude, coord.latitude) for coord in road.coordinates]
            geometry = LineString(coords)
            
            properties = {
                'id': road.road_id,
                'name': road.name,
                'type': road.road_type,
                'start_node_id': road.start_node_id,
                'end_node_id': road.end_node_id,
                'length': road.length,
                'speed_limit': road.speed_limit,
                'bidirectional': road.bidirectional
            }
            
            # 添加其他属性
            for key, value in road.properties.items():
                properties[key] = value
            
            road_data.append({**properties, 'geometry': geometry})
        
        # 创建GeoDataFrame
        if road_data:
            road_gdf = gpd.GeoDataFrame(road_data)
            road_gdf.to_file(file_path)
        else:
            warnings.warn("没有Road数据可保存")


"""
异常模块 - 定义地理空间感知系统中使用的自定义异常
"""


class GisenseError(Exception):
    """地理空间感知系统的基础异常类"""
    pass


class NodeNotFoundError(GisenseError):
    """当请求的节点在图中不存在时抛出"""
    
    def __init__(self, node_id, message=None):
        self.node_id = node_id
        self.message = message or f"节点不存在: {node_id}"
        super().__init__(self.message)


class EdgeNotFoundError(GisenseError):
    """当请求的边在图中不存在时抛出"""
    
    def __init__(self, start_node_id, end_node_id, message=None):
        self.start_node_id = start_node_id
        self.end_node_id = end_node_id
        self.message = message or f"边不存在: {start_node_id} -> {end_node_id}"
        super().__init__(self.message)


class PathNotFoundError(GisenseError):
    """当无法找到从起点到终点的路径时抛出"""
    
    def __init__(self, start_node_id, end_node_id, message=None):
        self.start_node_id = start_node_id
        self.end_node_id = end_node_id
        self.message = message or f"无法找到从 {start_node_id} 到 {end_node_id} 的路径"
        super().__init__(self.message)


class DataFormatError(GisenseError):
    """当输入数据格式不正确时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "数据格式错误"
        super().__init__(self.message)


class CoordinateError(GisenseError):
    """当坐标值无效或操作无效时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "坐标错误"
        super().__init__(self.message)


class MatchingError(GisenseError):
    """当坐标匹配失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "坐标匹配失败"
        super().__init__(self.message)


class VisualizationError(GisenseError):
    """当可视化操作失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "可视化错误"
        super().__init__(self.message)


class ConfigurationError(GisenseError):
    """当系统配置错误时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "配置错误"
        super().__init__(self.message)


class ObserverError(GisenseError):
    """当观察者操作失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "观察者错误"
        super().__init__(self.message)


class AlgorithmError(GisenseError):
    """当算法执行失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "算法执行错误"
        super().__init__(self.message)


class IOError(GisenseError):
    """当IO操作失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "IO错误"
        super().__init__(self.message)


class ValidationError(GisenseError):
    """当数据验证失败时抛出"""
    
    def __init__(self, message=None):
        self.message = message or "数据验证错误"
        super().__init__(self.message)


"""
图模块 - 提供地理空间网络的图表示
"""
from typing import Dict, List, Set, Tuple, Optional, Any, Iterator, Union
import json
import math
from collections import defaultdict, deque

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..utils.distance import haversine_distance
from ..utils.geometry import CoordinateMatcher
from .exceptions import NodeNotFoundError, EdgeNotFoundError, PathNotFoundError


class LocationGraph:
    """
    位置图类，表示地理空间网络
    
    这是系统的核心类，用于存储和管理地理空间网络的节点（POI）和边（道路）
    """
    
    def __init__(self):
        """初始化位置图"""
        # 存储节点 (POI)
        self.nodes: Dict[str, POI] = {}
        
        # 存储边 (Road)
        self.edges: Dict[str, Road] = {}
        
        # 邻接表，用于快速查找与节点相连的边
        self.adjacency: Dict[str, Dict[str, str]] = defaultdict(dict)
        
        # 坐标匹配器
        self.matcher: Optional[CoordinateMatcher] = None
        
        # 边的权重缓存 (用于路径规划)
        self._edge_weights: Dict[Tuple[str, str], float] = {}
    
    def add_node(self, node: POI) -> None:
        """
        添加节点到图中
        
        参数:
            node: POI对象
            
        异常:
            ValueError: 如果节点已存在
        """
        if node.poi_id in self.nodes:
            raise ValueError(f"节点已存在: {node.poi_id}")
        self.nodes[node.poi_id] = node
    
    def add_edge(self, edge: Road) -> None:
        """
        添加边到图中
        
        参数:
            edge: Road对象
        """
        # 存储边
        self.edges[edge.road_id] = edge
        
        # 更新邻接表
        start_node_id = edge.start_node_id
        end_node_id = edge.end_node_id
        
        # 添加正向边
        self.adjacency[start_node_id][end_node_id] = edge.road_id
        
        # 如果是双向道路，添加反向边
        if edge.bidirectional:
            self.adjacency[end_node_id][start_node_id] = edge.road_id
        
        # 更新节点的连接道路信息
        if start_node_id in self.nodes:
            self.nodes[start_node_id].add_connected_road(edge.road_id)
        
        if end_node_id in self.nodes:
            self.nodes[end_node_id].add_connected_road(edge.road_id)
        
        # 清除权重缓存
        self._edge_weights = {}
    
    def remove_node(self, node_id: str) -> bool:
        """
        从图中移除节点
        
        参数:
            node_id: 节点ID
            
        返回:
            如果成功移除则返回True，否则返回False
            
        注意:
            移除节点会同时移除与该节点相连的所有边
        """
        if node_id not in self.nodes:
            return False
        
        # 找出与该节点相连的所有边
        connected_edges = set()
        
        # 查找以该节点为起点的边
        for end_id, edge_id in list(self.adjacency[node_id].items()):
            connected_edges.add(edge_id)
        
        # 查找以该节点为终点的边
        for start_id, adj in list(self.adjacency.items()):
            if node_id in adj:
                connected_edges.add(adj[node_id])
        
        # 移除所有相连的边
        for edge_id in connected_edges:
            self.remove_edge(edge_id)
        
        # 移除节点
        del self.nodes[node_id]
        
        # 清除邻接表中的节点
        if node_id in self.adjacency:
            del self.adjacency[node_id]
        
        for start_id in list(self.adjacency.keys()):
            if node_id in self.adjacency[start_id]:
                del self.adjacency[start_id][node_id]
        
        return True
    
    def remove_edge(self, edge_id: str) -> bool:
        """
        从图中移除边
        
        参数:
            edge_id: 边ID
            
        返回:
            如果成功移除则返回True，否则返回False
        """
        if edge_id not in self.edges:
            return False
        
        edge = self.edges[edge_id]
        start_node_id = edge.start_node_id
        end_node_id = edge.end_node_id
        
        # 更新邻接表
        if start_node_id in self.adjacency and end_node_id in self.adjacency[start_node_id]:
            del self.adjacency[start_node_id][end_node_id]
        
        if edge.bidirectional and end_node_id in self.adjacency and start_node_id in self.adjacency[end_node_id]:
            del self.adjacency[end_node_id][start_node_id]
        
        # 更新节点的连接道路信息
        if start_node_id in self.nodes:
            self.nodes[start_node_id].remove_connected_road(edge_id)
        
        if end_node_id in self.nodes:
            self.nodes[end_node_id].remove_connected_road(edge_id)
        
        # 移除边
        del self.edges[edge_id]
        
        # 清除权重缓存
        self._edge_weights = {}
        
        return True
    
    def get_node(self, node_id: str) -> POI:
        """
        获取节点
        
        参数:
            node_id: 节点ID
            
        返回:
            POI对象
            
        异常:
            NodeNotFoundError: 如果节点不存在
        """
        if node_id not in self.nodes:
            raise NodeNotFoundError(node_id)
        
        return self.nodes[node_id]
    
    def get_edge(self, edge_id: str) -> Road:
        """
        获取边
        
        参数:
            edge_id: 边ID
            
        返回:
            Road对象
            
        异常:
            EdgeNotFoundError: 如果边不存在
        """
        if edge_id not in self.edges:
            raise EdgeNotFoundError(None, None, f"边不存在: {edge_id}")
        
        return self.edges[edge_id]
    
    def get_edge_between(self, start_node_id: str, end_node_id: str) -> Road:
        """
        获取连接两个节点的边
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            
        返回:
            Road对象
            
        异常:
            EdgeNotFoundError: 如果边不存在
        """
        if start_node_id not in self.adjacency or end_node_id not in self.adjacency[start_node_id]:
            raise EdgeNotFoundError(start_node_id, end_node_id)
        
        edge_id = self.adjacency[start_node_id][end_node_id]
        return self.edges[edge_id]
    
    def has_node(self, node_id: str) -> bool:
        """
        检查节点是否存在
        
        参数:
            node_id: 节点ID
            
        返回:
            如果节点存在则返回True，否则返回False
        """
        return node_id in self.nodes
    
    def has_edge(self, edge_id: str) -> bool:
        """
        检查边是否存在
        
        参数:
            edge_id: 边ID
            
        返回:
            如果边存在则返回True，否则返回False
        """
        return edge_id in self.edges
    
    def has_edge_between(self, start_node_id: str, end_node_id: str) -> bool:
        """
        检查两个节点之间是否存在边
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            
        返回:
            如果存在边则返回True，否则返回False
        """
        return start_node_id in self.adjacency and end_node_id in self.adjacency[start_node_id]
    
    def get_neighbors(self, node_id: str) -> Dict[str, str]:
        """
        获取节点的邻居节点
        
        参数:
            node_id: 节点ID
            
        返回:
            邻居节点ID到边ID的映射
            
        异常:
            NodeNotFoundError: 如果节点不存在
        """
        if node_id not in self.nodes:
            raise NodeNotFoundError(node_id)
        
        return dict(self.adjacency[node_id])
    
    def get_connected_edges(self, node_id: str) -> List[Road]:
        """
        获取与节点相连的所有边
        
        参数:
            node_id: 节点ID
            
        返回:
            Road对象列表
            
        异常:
            NodeNotFoundError: 如果节点不存在
        """
        if node_id not in self.nodes:
            raise NodeNotFoundError(node_id)
        
        connected_edges = []
        
        # 查找以该节点为起点的边
        for end_id, edge_id in self.adjacency[node_id].items():
            connected_edges.append(self.edges[edge_id])
        
        # 查找以该节点为终点的边 (单向边)
        for start_id, adj in self.adjacency.items():
            if start_id != node_id and node_id in adj:
                edge_id = adj[node_id]
                edge = self.edges[edge_id]
                if not edge.bidirectional:  # 只添加单向边，双向边已在上面添加
                    connected_edges.append(edge)
        
        return connected_edges
    
    def get_edge_weight(self, start_node_id: str, end_node_id: str, weight_type: str = 'distance') -> float:
        """
        获取边的权重
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            边的权重
            
        异常:
            EdgeNotFoundError: 如果边不存在
        """
        if start_node_id not in self.adjacency or end_node_id not in self.adjacency[start_node_id]:
            raise EdgeNotFoundError(start_node_id, end_node_id)
        
        # 检查缓存
        cache_key = (start_node_id, end_node_id, weight_type)
        if cache_key in self._edge_weights:
            return self._edge_weights[cache_key]
        
        edge_id = self.adjacency[start_node_id][end_node_id]
        edge = self.edges[edge_id]
        
        # 根据权重类型计算权重
        if weight_type == 'distance':
            weight = edge.length
        elif weight_type == 'time':
            weight = edge.get_travel_time() * 3600  # 转换为秒
        elif weight_type == 'custom':
            # 自定义权重，可以根据需要实现
            # 这里使用距离和时间的加权组合
            weight = edge.length + edge.get_travel_time() * 3600 * 0.1
        else:
            raise ValueError(f"未知的权重类型: {weight_type}")
        
        # 缓存权重
        self._edge_weights[cache_key] = weight
        
        return weight
    
    def find_path(self, start_node_id: str, end_node_id: str, 
                  algorithm: str = 'dijkstra', weight_type: str = 'distance') -> List[str]:
        """
        查找从起点到终点的最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            algorithm: 路径规划算法 ('dijkstra', 'a_star', 'bfs')
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            路径上的节点ID列表
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
            PathNotFoundError: 如果无法找到路径
        """
        if start_node_id not in self.nodes:
            raise NodeNotFoundError(start_node_id)
        
        if end_node_id not in self.nodes:
            raise NodeNotFoundError(end_node_id)
        
        # 如果起点和终点相同，直接返回
        if start_node_id == end_node_id:
            return [start_node_id]
        
        # 根据算法选择路径规划方法
        if algorithm == 'dijkstra':
            return self._dijkstra(start_node_id, end_node_id, weight_type)
        elif algorithm == 'a_star':
            return self._a_star(start_node_id, end_node_id, weight_type)
        elif algorithm == 'bfs':
            return self._bfs(start_node_id, end_node_id)
        else:
            raise ValueError(f"未知的路径规划算法: {algorithm}")
    
    def _dijkstra(self, start_node_id: str, end_node_id: str, weight_type: str = 'distance') -> List[str]:
        """
        使用Dijkstra算法查找最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 初始化
        distances = {node_id: float('inf') for node_id in self.nodes}
        distances[start_node_id] = 0
        previous = {node_id: None for node_id in self.nodes}
        unvisited = set(self.nodes.keys())
        
        while unvisited:
            # 找到距离最小的未访问节点
            current = min(unvisited, key=lambda node_id: distances[node_id])
            
            # 如果当前节点是终点，或者当前节点的距离是无穷大（无法到达），则结束
            if current == end_node_id or distances[current] == float('inf'):
                break
            
            # 移除当前节点
            unvisited.remove(current)
            
            # 更新邻居节点的距离
            for neighbor, edge_id in self.adjacency[current].items():
                if neighbor in unvisited:
                    weight = self.get_edge_weight(current, neighbor, weight_type)
                    new_distance = distances[current] + weight
                    
                    if new_distance < distances[neighbor]:
                        distances[neighbor] = new_distance
                        previous[neighbor] = current
        
        # 如果终点不可达
        if distances[end_node_id] == float('inf'):
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _a_star(self, start_node_id: str, end_node_id: str, weight_type: str = 'distance') -> List[str]:
        """
        使用A*算法查找最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 启发式函数：估计从当前节点到终点的距离
        def heuristic(node_id):
            # 使用直线距离作为启发式函数
            node = self.nodes[node_id]
            end_node = self.nodes[end_node_id]
            return haversine_distance(node.coordinate, end_node.coordinate)
        
        # 初始化
        open_set = {start_node_id}
        closed_set = set()
        
        g_score = {node_id: float('inf') for node_id in self.nodes}
        g_score[start_node_id] = 0
        
        f_score = {node_id: float('inf') for node_id in self.nodes}
        f_score[start_node_id] = heuristic(start_node_id)
        
        previous = {node_id: None for node_id in self.nodes}
        
        while open_set:
            # 找到f_score最小的节点
            current = min(open_set, key=lambda node_id: f_score[node_id])
            
            # 如果当前节点是终点，则结束
            if current == end_node_id:
                break
            
            # 移动当前节点
            open_set.remove(current)
            closed_set.add(current)
            
            # 更新邻居节点
            for neighbor, edge_id in self.adjacency[current].items():
                if neighbor in closed_set:
                    continue
                
                weight = self.get_edge_weight(current, neighbor, weight_type)
                tentative_g_score = g_score[current] + weight
                
                if neighbor not in open_set:
                    open_set.add(neighbor)
                elif tentative_g_score >= g_score[neighbor]:
                    continue
                
                previous[neighbor] = current
                g_score[neighbor] = tentative_g_score
                f_score[neighbor] = g_score[neighbor] + heuristic(neighbor)
        
        # 如果终点不可达
        if previous[end_node_id] is None and start_node_id != end_node_id:
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _bfs(self, start_node_id: str, end_node_id: str) -> List[str]:
        """
        使用广度优先搜索查找路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 初始化
        queue = deque([start_node_id])
        visited = {start_node_id}
        previous = {start_node_id: None}
        
        while queue:
            current = queue.popleft()
            
            # 如果当前节点是终点，则结束
            if current == end_node_id:
                break
            
            # 遍历邻居节点
            for neighbor in self.adjacency[current]:
                if neighbor not in visited:
                    queue.append(neighbor)
                    visited.add(neighbor)
                    previous[neighbor] = current
        
        # 如果终点不可达
        if end_node_id not in previous:
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def get_path_edges(self, path: List[str]) -> List[Road]:
        """
        获取路径上的边
        
        参数:
            path: 路径上的节点ID列表
            
        返回:
            路径上的Road对象列表
            
        异常:
            EdgeNotFoundError: 如果路径上的某两个节点之间不存在边
        """
        if len(path) < 2:
            return []
        
        edges = []
        
        for i in range(len(path) - 1):
            start_node_id = path[i]
            end_node_id = path[i + 1]
            
            if not self.has_edge_between(start_node_id, end_node_id):
                raise EdgeNotFoundError(start_node_id, end_node_id)
            
            edge_id = self.adjacency[start_node_id][end_node_id]
            edges.append(self.edges[edge_id])
        
        return edges
    
    def get_path_coordinates(self, path: List[str]) -> List[Coordinate]:
        """
        获取路径上的坐标点
        
        参数:
            path: 路径上的节点ID列表
            
        返回:
            路径上的坐标点列表
        """
        if not path:
            return []
        
        coordinates = []
        
        # 添加起点坐标
        start_node = self.nodes[path[0]]
        coordinates.append(start_node.coordinate)
        
        # 添加路径上的边的坐标
        for i in range(len(path) - 1):
            start_node_id = path[i]
            end_node_id = path[i + 1]
            
            edge_id = self.adjacency[start_node_id][end_node_id]
            edge = self.edges[edge_id]
            
            # 添加边上的中间点（不包括起点）
            coordinates.extend(edge.coordinates[1:])
        
        return coordinates
    
    def get_path_length(self, path: List[str]) -> float:
        """
        计算路径的总长度
        
        参数:
            path: 路径上的节点ID列表
            
        返回:
            路径总长度 (米)
        """
        if len(path) < 2:
            return 0.0
        
        total_length = 0.0
        
        for i in range(len(path) - 1):
            start_node_id = path[i]
            end_node_id = path[i + 1]
            
            edge_id = self.adjacency[start_node_id][end_node_id]
            edge = self.edges[edge_id]
            
            total_length += edge.length
        
        return total_length
    
    def get_path_travel_time(self, path: List[str]) -> float:
        """
        计算路径的理论通行时间
        
        参数:
            path: 路径上的节点ID列表
            
        返回:
            通行时间 (小时)
        """
        if len(path) < 2:
            return 0.0
        
        total_time = 0.0
        
        for i in range(len(path) - 1):
            start_node_id = path[i]
            end_node_id = path[i + 1]
            
            edge_id = self.adjacency[start_node_id][end_node_id]
            edge = self.edges[edge_id]
            
            total_time += edge.get_travel_time()
        
        return total_time
    
    def find_nodes_in_range(self, center: Coordinate, radius: float) -> List[Tuple[POI, float]]:
        """
        查找指定范围内的节点
        
        参数:
            center: 中心坐标
            radius: 半径 (米)
            
        返回:
            (POI, 距离) 元组的列表，按距离升序排序
        """
        result = []
        
        for node in self.nodes.values():
            distance = haversine_distance(center, node.coordinate)
            if distance <= radius:
                result.append((node, distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[1])
    
    def find_nearest_node(self, coord: Coordinate) -> Tuple[POI, float]:
        """
        查找最近的节点
        
        参数:
            coord: 坐标
            
        返回:
            (最近的POI, 距离) 元组
            
        异常:
            ValueError: 如果图中没有节点
        """
        if not self.nodes:
            raise ValueError("图中没有节点")
        
        min_distance = float('inf')
        nearest_node = None
        
        for node in self.nodes.values():
            distance = haversine_distance(coord, node.coordinate)
            if distance < min_distance:
                min_distance = distance
                nearest_node = node
        
        return nearest_node, min_distance
    
    def match_coordinate(self, coord: Coordinate) -> Dict[str, Any]:
        """
        将坐标匹配到最近的道路上
        
        参数:
            coord: 要匹配的坐标
            
        返回:
            包含匹配结果的字典，如果没有匹配到则返回None
        """
        # 如果没有初始化匹配器，则初始化
        if self.matcher is None:
            self.matcher = CoordinateMatcher(list(self.edges.values()))
        
        return self.matcher.match_coordinate(coord)
    
    def match_coordinates(self, coords: List[Coordinate], max_gap: int = 5) -> List[Dict[str, Any]]:
        """
        将一系列坐标匹配到道路上
        
        参数:
            coords: 要匹配的坐标列表
            max_gap: 允许的最大连续未匹配点数
            
        返回:
            匹配结果列表
        """
        # 如果没有初始化匹配器，则初始化
        if self.matcher is None:
            self.matcher = CoordinateMatcher(list(self.edges.values()))
        
        return self.matcher.match_coordinates(coords, max_gap)
    
    def rebuild_matcher(self) -> None:
        """重建坐标匹配器"""
        self.matcher = CoordinateMatcher(list(self.edges.values()))
    
    def get_subgraph(self, node_ids: Set[str]) -> 'LocationGraph':
        """
        获取子图
        
        参数:
            node_ids: 子图中包含的节点ID集合
            
        返回:
            LocationGraph对象
        """
        subgraph = LocationGraph()
        
        # 添加节点
        for node_id in node_ids:
            if node_id in self.nodes:
                subgraph.add_node(self.nodes[node_id])
        
        # 添加边
        for node_id in node_ids:
            if node_id in self.adjacency:
                for end_id, edge_id in self.adjacency[node_id].items():
                    if end_id in node_ids:
                        subgraph.add_edge(self.edges[edge_id])
        
        return subgraph
    
    def get_connected_components(self) -> List[Set[str]]:
        """
        获取图的连通分量
        
        返回:
            连通分量列表，每个连通分量是一个节点ID集合
        """
        components = []
        visited = set()
        
        for node_id in self.nodes:
            if node_id not in visited:
                # 使用BFS查找连通分量
                component = set()
                queue = deque([node_id])
                component.add(node_id)
                visited.add(node_id)
                
                while queue:
                    current = queue.popleft()
                    
                    for neighbor in self.adjacency[current]:
                        if neighbor not in visited:
                            queue.append(neighbor)
                            component.add(neighbor)
                            visited.add(neighbor)
                
                components.append(component)
        
        return components
    
    def get_statistics(self) -> Dict[str, Any]:
        """
        获取图的统计信息
        
        返回:
            包含统计信息的字典
        """
        # 计算总道路长度
        total_length = sum(edge.length for edge in self.edges.values())
        
        # 计算节点的度分布
        degree_dist = defaultdict(int)
        for node_id in self.nodes:
            degree = len(self.adjacency[node_id])
            degree_dist[degree] += 1
        
        # 计算连通分量
        components = self.get_connected_components()
        
        return {
            'node_count': len(self.nodes),
            'edge_count': len(self.edges),
            'total_length': total_length,
            'degree_distribution': dict(degree_dist),
            'component_count': len(components),
            'largest_component_size': max(len(comp) for comp in components) if components else 0
        }


"""
地图可视化模块 - 提供地理空间数据的可视化功能
"""
from typing import List, Dict, Any, Tuple, Optional, Union
import math
import os
import warnings

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..core.graph import LocationGraph
from ..core.observer import Observer
from ..core.exceptions import VisualizationError


class MapVisualizer:
    """
    地图可视化器，用于可视化地理空间数据
    
    注意：此类依赖于matplotlib库，需要先安装：
    pip install matplotlib
    """
    
    def __init__(self, figsize: Tuple[int, int] = (10, 8), dpi: int = 100):
        """
        初始化地图可视化器
        
        参数:
            figsize: 图形大小 (宽, 高)，单位为英寸
            dpi: 图像分辨率 (每英寸点数)
        """
        try:
            import matplotlib.pyplot as plt
            self.plt = plt
        except ImportError:
            raise ImportError("未安装matplotlib库，请先安装: pip install matplotlib")
        
        self.figsize = figsize
        self.dpi = dpi
        self.fig = None
        self.ax = None
        self.colormap = {
            'motorway': '#FF6347',      # 红色
            'trunk': '#FF8C00',         # 深橙色
            'primary': '#FFA500',       # 橙色
            'secondary': '#FFD700',     # 金色
            'tertiary': '#FFFF00',      # 黄色
            'residential': '#ADFF2F',   # 绿黄色
            'service': '#98FB98',       # 淡绿色
            'pedestrian': '#87CEFA',    # 淡蓝色
            'footway': '#1E90FF',       # 道奇蓝
            'cycleway': '#9370DB',      # 中紫色
            'path': '#D3D3D3',          # 淡灰色
            'junction': '#A9A9A9',      # 深灰色
            'generic': '#808080'        # 灰色
        }
        self.node_colors = {
            'restaurant': '#FF6347',    # 红色
            'cafe': '#8B4513',          # 棕色
            'shop': '#FF8C00',          # 深橙色
            'supermarket': '#FFA500',   # 橙色
            'school': '#FFD700',        # 金色
            'hospital': '#FF0000',      # 鲜红色
            'pharmacy': '#FF69B4',      # 粉红色
            'bank': '#32CD32',          # 酸橙绿
            'park': '#228B22',          # 森林绿
            'cinema': '#4B0082',        # 靛青色
            'hotel': '#1E90FF',         # 道奇蓝
            'fuel': '#B8860B',          # 暗金色
            'parking': '#708090',       # 板岩灰
            'bus_stop': '#9370DB',      # 中紫色
            'railway_station': '#800000', # 栗色
            'junction': '#A9A9A9',      # 深灰色
            'generic': '#808080'        # 灰色
        }
    
    def create_figure(self) -> None:
        """创建新的图形"""
        self.fig, self.ax = self.plt.subplots(figsize=self.figsize, dpi=self.dpi)
        self.ax.set_aspect('equal')
        self.ax.grid(True, linestyle='--', alpha=0.7)
    
    def plot_graph(self, graph: LocationGraph, show_node_labels: bool = False, 
                  show_edge_labels: bool = False, node_size: int = 30, 
                  edge_width: float = 1.0) -> None:
        """
        绘制地理空间图
        
        参数:
            graph: 地理空间图
            show_node_labels: 是否显示节点标签
            show_edge_labels: 是否显示边标签
            node_size: 节点大小
            edge_width: 边宽度
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 清除现有图形元素
        self.ax.clear()
        
        # 绘制边 (道路)
        edge_lines = []
        for edge in graph.edges.values():
            # 提取坐标
            lons = [coord.longitude for coord in edge.coordinates]
            lats = [coord.latitude for coord in edge.coordinates]
            
            # 确定颜色
            color = self.colormap.get(edge.road_type, self.colormap['generic'])
            
            # 绘制线段
            line = self.ax.plot(lons, lats, color=color, linewidth=edge_width, alpha=0.8)
            edge_lines.append(line[0])
            
            # 显示边标签
            if show_edge_labels and edge.name:
                # 计算线段中点
                mid_idx = len(edge.coordinates) // 2
                mid_lon = edge.coordinates[mid_idx].longitude
                mid_lat = edge.coordinates[mid_idx].latitude
                
                self.ax.text(mid_lon, mid_lat, edge.name, fontsize=8, 
                           ha='center', va='center', alpha=0.7,
                           bbox=dict(boxstyle="round,pad=0.3", fc="white", ec="gray", alpha=0.7))
        
        # 绘制节点 (POI)
        node_points = []
        for node in graph.nodes.values():
            # 提取坐标
            lon = node.coordinate.longitude
            lat = node.coordinate.latitude
            
            # 确定颜色
            color = self.node_colors.get(node.poi_type, self.node_colors['generic'])
            
            # 绘制点
            point = self.ax.scatter(lon, lat, color=color, s=node_size, alpha=0.8, zorder=2)
            node_points.append(point)
            
            # 显示节点标签
            if show_node_labels and node.name:
                self.ax.text(lon, lat + 0.0001, node.name, fontsize=8, 
                           ha='center', va='bottom', alpha=0.9,
                           bbox=dict(boxstyle="round,pad=0.2", fc="white", ec="gray", alpha=0.7))
        
        # 设置标题和轴标签
        self.ax.set_title('Geographic Network Visualization')
        self.ax.set_xlabel('Longitude')
        self.ax.set_ylabel('Latitude')
        self.ax.grid(True, linestyle='--', alpha=0.7)
        self.ax.set_aspect('equal')
    
    def plot_path(self, path_coords: List[Coordinate], color: str = 'blue', 
                linewidth: float = 2.0, alpha: float = 1.0, label: str = 'Path') -> None:
        """
        绘制路径
        
        参数:
            path_coords: 路径坐标列表
            color: 路径颜色
            linewidth: 线宽
            alpha: 透明度
            label: 图例标签
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 提取坐标
        lons = [coord.longitude for coord in path_coords]
        lats = [coord.latitude for coord in path_coords]
        
        # 绘制路径
        self.ax.plot(lons, lats, color=color, linewidth=linewidth, alpha=alpha, label=label, zorder=3)
        
        # 绘制起点和终点
        if path_coords:
            # 起点 (绿色三角形)
            self.ax.scatter(path_coords[0].longitude, path_coords[0].latitude, 
                          color='green', marker='^', s=100, alpha=0.8, zorder=4)
            
            # 终点 (红色星形)
            self.ax.scatter(path_coords[-1].longitude, path_coords[-1].latitude, 
                          color='red', marker='*', s=150, alpha=0.8, zorder=4)
    
    def plot_multiple_paths(self, paths: List[List[Coordinate]], 
                          colors: Optional[List[str]] = None, 
                          labels: Optional[List[str]] = None) -> None:
        """
        绘制多条路径
        
        参数:
            paths: 路径坐标列表的列表
            colors: 路径颜色列表
            labels: 图例标签列表
        """
        if not paths:
            return
        
        if colors is None:
            # 默认颜色列表
            default_colors = ['blue', 'red', 'green', 'purple', 'orange', 'brown', 'pink', 'gray', 'olive', 'cyan']
            colors = [default_colors[i % len(default_colors)] for i in range(len(paths))]
        
        if labels is None:
            labels = [f'Path {i+1}' for i in range(len(paths))]
        
        for i, path in enumerate(paths):
            color = colors[i % len(colors)]
            label = labels[i] if i < len(labels) else f'Path {i+1}'
            self.plot_path(path, color=color, label=label)
    
    def plot_observer(self, observer: Union[Observer, Coordinate], color: str = 'blue',
                    marker: str = 'o', size: int = 100) -> None:
        """
        绘制观察者或坐标点
        
        参数:
            observer: 观察者对象或坐标对象
            color: 颜色
            marker: 标记
            size: 大小
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 处理Coordinate输入
        if isinstance(observer, Coordinate):
            self.ax.scatter(observer.longitude, observer.latitude,
                          color=color, marker=marker, s=size, alpha=0.8, zorder=5)
            return
            
        # 处理Observer输入
        # 绘制观察者位置
        self.ax.scatter(observer.location.longitude, observer.location.latitude,
                      color=color, marker=marker, s=size, alpha=0.8, zorder=5)
        
        # 绘制观察者朝向
        heading_rad = math.radians(observer.heading)
        arrow_length = 0.0005  # 箭头长度
        dx = arrow_length * math.sin(heading_rad)
        dy = arrow_length * math.cos(heading_rad)
        
        self.ax.arrow(observer.location.longitude, observer.location.latitude,
                    dx, dy, head_width=0.0001, head_length=0.0002,
                    fc=color, ec=color, zorder=5)
        
        # 绘制观察者视野范围
        if observer.fov < 360:
            # 计算视野扇形的起始和结束角度
            start_angle = observer.heading - observer.fov / 2
            end_angle = observer.heading + observer.fov / 2
            
            # 生成扇形点
            theta = [math.radians(angle) for angle in range(int(start_angle), int(end_angle) + 1)]
            
            # 计算扇形边缘点的坐标
            edge_x = [observer.location.longitude + observer.perception_range * 0.00001 * math.sin(angle) for angle in theta]
            edge_y = [observer.location.latitude + observer.perception_range * 0.00001 * math.cos(angle) for angle in theta]
            
            # 添加观察者位置作为扇形的起点
            x = [observer.location.longitude] + edge_x
            y = [observer.location.latitude] + edge_y
            
            # 绘制扇形
            self.ax.fill(x, y, color=color, alpha=0.2, zorder=4)
        else:
            # 绘制圆形感知范围
            circle = self.plt.Circle((observer.location.longitude, observer.location.latitude),
                                   observer.perception_range * 0.00001,
                                   color=color, alpha=0.2, zorder=4)
            self.ax.add_patch(circle)
    
    def plot_trajectory(self, trajectory: List[Coordinate], color: str = 'blue', 
                      linewidth: float = 1.5, alpha: float = 0.8, 
                      show_points: bool = True, point_size: int = 20) -> None:
        """
        绘制轨迹
        
        参数:
            trajectory: 轨迹坐标列表
            color: 轨迹颜色
            linewidth: 线宽
            alpha: 透明度
            show_points: 是否显示轨迹点
            point_size: 轨迹点大小
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 提取坐标
        lons = [coord.longitude for coord in trajectory]
        lats = [coord.latitude for coord in trajectory]
        
        # 绘制轨迹线
        self.ax.plot(lons, lats, color=color, linewidth=linewidth, alpha=alpha, zorder=3)
        
        # 绘制轨迹点
        if show_points:
            self.ax.scatter(lons, lats, color=color, s=point_size, alpha=alpha, zorder=3)
    
    def plot_heatmap(self, points: List[Coordinate], intensity: Optional[List[float]] = None, 
                    radius: int = 10) -> None:
        """
        绘制热力图
        
        参数:
            points: 点坐标列表
            intensity: 点强度列表，如果为None则所有点强度相同
            radius: 热力图半径
        """
        try:
            from matplotlib.colors import LinearSegmentedColormap
        except ImportError:
            raise ImportError("未安装matplotlib库，请先安装: pip install matplotlib")
        
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 提取坐标
        lons = [point.longitude for point in points]
        lats = [point.latitude for point in points]
        
        # 如果没有提供强度，则所有点强度相同
        if intensity is None:
            intensity = [1.0] * len(points)
        
        # 创建热力图
        heatmap = self.ax.hexbin(lons, lats, C=intensity, gridsize=50, cmap='hot', alpha=0.6, zorder=1)
        
        # 添加颜色条
        self.fig.colorbar(heatmap, ax=self.ax, label='Intensity')
    
    def plot_area(self, polygon: List[Coordinate], color: str = 'blue', 
                alpha: float = 0.3, label: Optional[str] = None) -> None:
        """
        绘制区域
        
        参数:
            polygon: 多边形顶点坐标列表
            color: 填充颜色
            alpha: 透明度
            label: 图例标签
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        # 提取坐标
        lons = [point.longitude for point in polygon]
        lats = [point.latitude for point in polygon]
        
        # 确保多边形闭合
        if polygon[0] != polygon[-1]:
            lons.append(polygon[0].longitude)
            lats.append(polygon[0].latitude)
        
        # 绘制多边形
        self.ax.fill(lons, lats, color=color, alpha=alpha, label=label, zorder=2)
        self.ax.plot(lons, lats, color=color, alpha=alpha+0.2, zorder=2)
    
    def plot_events(self, events: List[Dict[str, Any]], event_type: str, 
                  marker: str = 'o', color: str = 'red', size: int = 100) -> None:
        """
        绘制事件
        
        参数:
            events: 事件列表
            event_type: 事件类型 ('turn', 'stop', 'junction_crossing')
            marker: 事件标记
            color: 事件颜色
            size: 事件大小
        """
        if self.fig is None or self.ax is None:
            self.create_figure()
        
        for event in events:
            if 'coordinate' in event:
                coord = event['coordinate']
                self.ax.scatter(coord.longitude, coord.latitude, 
                              color=color, marker=marker, s=size, alpha=0.8, zorder=5)
                
                # 添加事件标签
                if event_type == 'turn' and 'direction' in event:
                    label = f"{event['direction']} ({event['angle']:.1f}°)"
                    self.ax.text(coord.longitude, coord.latitude + 0.0001, label, 
                               fontsize=8, ha='center', va='bottom', alpha=0.9,
                               bbox=dict(boxstyle="round,pad=0.2", fc="white", ec="gray", alpha=0.7))
                
                elif event_type == 'stop' and 'duration' in event:
                    label = f"Stop ({event['duration']:.1f}s)"
                    self.ax.text(coord.longitude, coord.latitude + 0.0001, label, 
                               fontsize=8, ha='center', va='bottom', alpha=0.9,
                               bbox=dict(boxstyle="round,pad=0.2", fc="white", ec="gray", alpha=0.7))
                
                elif event_type == 'junction_crossing' and 'junction' in event:
                    junction = event['junction']
                    label = f"Junction: {junction.name or junction.poi_id}"
                    self.ax.text(coord.longitude, coord.latitude + 0.0001, label, 
                               fontsize=8, ha='center', va='bottom', alpha=0.9,
                               bbox=dict(boxstyle="round,pad=0.2", fc="white", ec="gray", alpha=0.7))
    
    def add_legend(self) -> None:
        """添加图例"""
        if self.fig is None or self.ax is None:
            return
        
        self.ax.legend(loc='best')
    
    def add_scale_bar(self, length_km: float = 1.0, location: str = 'lower right') -> None:
        """
        添加比例尺
        
        参数:
            length_km: 比例尺长度 (千米)
            location: 比例尺位置
        """
        if self.fig is None or self.ax is None:
            return
        
        # 获取当前坐标范围
        xmin, xmax = self.ax.get_xlim()
        ymin, ymax = self.ax.get_ylim()
        
        # 计算比例尺在经度上的长度
        # 在赤道上，1度经度约等于111.32千米
        # 在纬度lat上，1度经度约等于111.32 * cos(lat)千米
        lat_avg = (ymin + ymax) / 2
        lon_length = length_km / (111.32 * math.cos(math.radians(lat_avg)))
        
        # 计算比例尺位置
        if location == 'lower right':
            x_start = xmax - lon_length - (xmax - xmin) * 0.05
            y_pos = ymin + (ymax - ymin) * 0.05
        elif location == 'lower left':
            x_start = xmin + (xmax - xmin) * 0.05
            y_pos = ymin + (ymax - ymin) * 0.05
        elif location == 'upper right':
            x_start = xmax - lon_length - (xmax - xmin) * 0.05
            y_pos = ymax - (ymax - ymin) * 0.05
        elif location == 'upper left':
            x_start = xmin + (xmax - xmin) * 0.05
            y_pos = ymax - (ymax - ymin) * 0.05
        else:
            x_start = xmin + (xmax - xmin) * 0.05
            y_pos = ymin + (ymax - ymin) * 0.05
        
        # 绘制比例尺
        x_end = x_start + lon_length
        self.ax.plot([x_start, x_end], [y_pos, y_pos], 'k-', linewidth=2)
        self.ax.plot([x_start, x_start], [y_pos - (ymax - ymin) * 0.005, y_pos + (ymax - ymin) * 0.005], 'k-', linewidth=2)
        self.ax.plot([x_end, x_end], [y_pos - (ymax - ymin) * 0.005, y_pos + (ymax - ymin) * 0.005], 'k-', linewidth=2)
        
        # 添加标签
        self.ax.text((x_start + x_end) / 2, y_pos + (ymax - ymin) * 0.01, f'{length_km} km', 
                   ha='center', va='bottom', fontsize=8)
    
    def add_north_arrow(self, location: str = 'upper right') -> None:
        """
        添加指北针
        
        参数:
            location: 指北针位置
        """
        if self.fig is None or self.ax is None:
            return
        
        # 获取当前坐标范围
        xmin, xmax = self.ax.get_xlim()
        ymin, ymax = self.ax.get_ylim()
        
        # 计算指北针位置
        if location == 'upper right':
            x_pos = xmax - (xmax - xmin) * 0.05
            y_pos = ymax - (ymax - ymin) * 0.05
        elif location == 'upper left':
            x_pos = xmin + (xmax - xmin) * 0.05
            y_pos = ymax - (ymax - ymin) * 0.05
        elif location == 'lower right':
            x_pos = xmax - (xmax - xmin) * 0.05
            y_pos = ymin + (ymax - ymin) * 0.05
        elif location == 'lower left':
            x_pos = xmin + (xmax - xmin) * 0.05
            y_pos = ymin + (ymax - ymin) * 0.05
        else:
            x_pos = xmax - (xmax - xmin) * 0.05
            y_pos = ymax - (ymax - ymin) * 0.05
        
        # 绘制指北针
        arrow_length = (ymax - ymin) * 0.05
        self.ax.arrow(x_pos, y_pos - arrow_length, 0, arrow_length, 
                    head_width=(xmax - xmin) * 0.01, head_length=arrow_length * 0.3, 
                    fc='k', ec='k')
        
        # 添加标签
        self.ax.text(x_pos, y_pos + arrow_length * 0.1, 'N', 
                   ha='center', va='bottom', fontsize=10, fontweight='bold')
    
    def save_figure(self, file_path: str, dpi: Optional[int] = None) -> None:
        """
        保存图形到文件
        
        参数:
            file_path: 文件路径
            dpi: 图像分辨率，如果为None则使用初始化时设置的dpi
        """
        if self.fig is None:
            raise VisualizationError("没有可保存的图形")
        
        if dpi is None:
            dpi = self.dpi
        
        try:
            # 创建目录（如果不存在）
            directory = os.path.dirname(file_path)
            if directory and not os.path.exists(directory):
                os.makedirs(directory)
            
            # 保存图形
            self.fig.savefig(file_path, dpi=dpi, bbox_inches='tight')
            print(f"图形已保存到: {file_path}")
        
        except Exception as e:
            raise VisualizationError(f"保存图形失败: {str(e)}")
    
    def show(self) -> None:
        """显示图形"""
        if self.fig is None:
            raise VisualizationError("没有可显示的图形")
        
        self.plt.show()
    
    def close(self) -> None:
        """关闭图形"""
        if self.fig is not None:
            self.plt.close(self.fig)
            self.fig = None
            self.ax = None


class InteractiveMapVisualizer(MapVisualizer):
    """
    交互式地图可视化器，提供交互式地图可视化功能
    
    注意：此类依赖于ipywidgets库，需要先安装：
    pip install ipywidgets
    """
    
    def __init__(self, figsize: Tuple[int, int] = (10, 8), dpi: int = 100):
        """
        初始化交互式地图可视化器
        
        参数:
            figsize: 图形大小 (宽, 高)，单位为英寸
            dpi: 图像分辨率 (每英寸点数)
        """
        super().__init__(figsize, dpi)
        
        try:
            import ipywidgets as widgets
            self.widgets = widgets
        except ImportError:
            warnings.warn("未安装ipywidgets库，交互功能将不可用。请安装: pip install ipywidgets")
            self.widgets = None
    
    def create_interactive_map(self, graph: LocationGraph) -> Any:
        """
        创建交互式地图
        
        参数:
            graph: 地理空间图
            
        返回:
            交互式地图小部件
        """
        if self.widgets is None:
            raise ImportError("未安装ipywidgets库，请先安装: pip install ipywidgets")
        
        # 创建图形
        self.create_figure()
        
        # 绘制图
        self.plot_graph(graph, show_node_labels=False, show_edge_labels=False)
        
        # 创建控件
        show_nodes = self.widgets.Checkbox(value=True, description='显示节点')
        show_node_labels = self.widgets.Checkbox(value=False, description='显示节点标签')
        show_edges = self.widgets.Checkbox(value=True, description='显示边')
        show_edge_labels = self.widgets.Checkbox(value=False, description='显示边标签')
        node_size = self.widgets.IntSlider(value=30, min=5, max=100, step=5, description='节点大小')
        edge_width = self.widgets.FloatSlider(value=1.0, min=0.5, max=5.0, step=0.5, description='边宽度')
        
        # 更新函数
        def update(show_nodes, show_node_labels, show_edges, show_edge_labels, node_size, edge_width):
            self.ax.clear()
            
            # 绘制边
            if show_edges:
                for edge in graph.edges.values():
                    lons = [coord.longitude for coord in edge.coordinates]
                    lats = [coord.latitude for coord in edge.coordinates]
                    color = self.colormap.get(edge.road_type, self.colormap['generic'])
                    self.ax.plot(lons, lats, color=color, linewidth=edge_width, alpha=0.8)
                    
                    if show_edge_labels and edge.name:
                        mid_idx = len(edge.coordinates) // 2
                        mid_lon = edge.coordinates[mid_idx].longitude
                        mid_lat = edge.coordinates[mid_idx].latitude
                        self.ax.text(mid_lon, mid_lat, edge.name, fontsize=8, 
                                   ha='center', va='center', alpha=0.7,
                                   bbox=dict(boxstyle="round,pad=0.3", fc="white", ec="gray", alpha=0.7))
            
            # 绘制节点
            if show_nodes:
                for node in graph.nodes.values():
                    lon = node.coordinate.longitude
                    lat = node.coordinate.latitude
                    color = self.node_colors.get(node.poi_type, self.node_colors['generic'])
                    self.ax.scatter(lon, lat, color=color, s=node_size, alpha=0.8, zorder=2)
                    
                    if show_node_labels and node.name:
                        self.ax.text(lon, lat + 0.0001, node.name, fontsize=8, 
                                   ha='center', va='bottom', alpha=0.9,
                                   bbox=dict(boxstyle="round,pad=0.2", fc="white", ec="gray", alpha=0.7))
            
            # 设置标题和轴标签
            self.ax.set_title('Interactive Geographic Network Visualization')
            self.ax.set_xlabel('Longitude')
            self.ax.set_ylabel('Latitude')
            self.ax.grid(True, linestyle='--', alpha=0.7)
            self.ax.set_aspect('equal')
            
            # 刷新图形
            self.fig.canvas.draw_idle()
        
        # 创建交互式小部件
        interactive_map = self.widgets.interactive(
            update,
            show_nodes=show_nodes,
            show_node_labels=show_node_labels,
            show_edges=show_edges,
            show_edge_labels=show_edge_labels,
            node_size=node_size,
            edge_width=edge_width
        )
        
        return interactive_map
    
    def create_path_explorer(self, graph: LocationGraph, pathfinder) -> Any:
        """
        创建路径探索器
        
        参数:
            graph: 地理空间图
            pathfinder: 路径规划器对象
            
        返回:
            路径探索器小部件
        """
        if self.widgets is None:
            raise ImportError("未安装ipywidgets库，请先安装: pip install ipywidgets")
        
        # 创建图形
        self.create_figure()
        
        # 绘制图
        self.plot_graph(graph, show_node_labels=False, show_edge_labels=False)
        
        # 获取所有节点ID和名称
        node_options = [(f"{node.name} ({node.poi_id})" if node.name else node.poi_id) for node in graph.nodes.values()]
        node_ids = [node.poi_id for node in graph.nodes.values()]
        
        # 创建控件
        start_dropdown = self.widgets.Dropdown(options=node_options, description='起点:')
        end_dropdown = self.widgets.Dropdown(options=node_options, description='终点:')
        algorithm_dropdown = self.widgets.Dropdown(
            options=['dijkstra', 'a_star', 'bfs'],
            value='dijkstra',
            description='算法:'
        )
        weight_dropdown = self.widgets.Dropdown(
            options=['distance', 'time', 'custom'],
            value='distance',
            description='权重:'
        )
        find_button = self.widgets.Button(description='查找路径')
        clear_button = self.widgets.Button(description='清除路径')
        output = self.widgets.Output()
        
        # 更新函数
        def on_find_button_clicked(b):
            with output:
                output.clear_output()
                
                try:
                    start_idx = start_dropdown.index
                    end_idx = end_dropdown.index
                    start_id = node_ids[start_idx]
                    end_id = node_ids[end_idx]
                    algorithm = algorithm_dropdown.value
                    weight_type = weight_dropdown.value
                    
                    print(f"查找从 {start_id} 到 {end_id} 的路径，使用 {algorithm} 算法和 {weight_type} 权重...")
                    
                    # 查找路径
                    path_result = pathfinder.find_shortest_path(start_id, end_id, weight_type)
                    
                    # 清除之前的路径
                    for line in self.ax.lines[:]:
                        if line.get_label() == 'Path':
                            line.remove()
                    
                    # 绘制新路径
                    self.plot_path(path_result['coordinates'])
                    
                    # 显示路径信息
                    print(f"路径长度: {path_result['total_distance']:.1f} 米")
                    print(f"预计时间: {path_result['total_time'] * 60:.1f} 分钟")
                    print(f"节点数量: {len(path_result['path'])}")
                    print(f"边数量: {len(path_result['edges'])}")
                    
                    # 刷新图形
                    self.fig.canvas.draw_idle()
                
                except Exception as e:
                    print(f"错误: {str(e)}")
        
        def on_clear_button_clicked(b):
            with output:
                output.clear_output()
                
                # 清除路径
                for line in self.ax.lines[:]:
                    if line.get_label() == 'Path':
                        line.remove()
                
                # 刷新图形
                self.fig.canvas.draw_idle()
                
                print("路径已清除")
        
        # 绑定按钮事件
        find_button.on_click(on_find_button_clicked)
        clear_button.on_click(on_clear_button_clicked)
        
        # 创建布局
        controls = self.widgets.VBox([
            self.widgets.HBox([start_dropdown, end_dropdown]),
            self.widgets.HBox([algorithm_dropdown, weight_dropdown]),
            self.widgets.HBox([find_button, clear_button]),
            output
        ])
        
        return controls


"""
路径规划模块 - 提供各种路径规划算法
"""
from typing import List, Dict, Any, Tuple, Set, Optional, Callable
import heapq
import math
from collections import defaultdict, deque

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..core.graph import LocationGraph
from ..core.exceptions import PathNotFoundError, NodeNotFoundError
from ..utils.distance import haversine_distance


class PathFinder:
    """
    路径规划器，提供各种路径规划算法
    """
    
    def __init__(self, graph: LocationGraph):
        """
        初始化路径规划器
        
        参数:
            graph: 地理空间图
        """
        self.graph = graph
    
    def find_shortest_path(self, start_node_id: str, end_node_id: str, 
                          weight_type: str = 'distance', weight_func=None) -> Dict[str, Any]:
        """
        使用Dijkstra算法查找最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            包含路径信息的字典:
            {
                'path': 路径上的节点ID列表,
                'edges': 路径上的边对象列表,
                'total_distance': 路径总长度 (米),
                'total_time': 路径总时间 (小时),
                'coordinates': 路径上的坐标点列表
            }
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
            PathNotFoundError: 如果无法找到路径
        """
        # 检查节点是否存在
        if not self.graph.has_node(start_node_id):
            raise NodeNotFoundError(start_node_id)
        
        if not self.graph.has_node(end_node_id):
            raise NodeNotFoundError(end_node_id)
        
        # 如果起点和终点相同，直接返回
        if start_node_id == end_node_id:
            start_node = self.graph.get_node(start_node_id)
            return {
                'path': [start_node_id],
                'edges': [],
                'total_distance': 0.0,
                'total_time': 0.0,
                'coordinates': [start_node.coordinate]
            }
        
        # 使用Dijkstra算法查找最短路径
        path = self._dijkstra(start_node_id, end_node_id, weight_type, weight_func)
        
        # 获取路径上的边
        edges = self.graph.get_path_edges(path)
        
        # 计算路径总长度和时间
        total_distance = sum(edge.length for edge in edges)
        total_time = sum(edge.get_travel_time() for edge in edges)
        
        # 获取路径上的坐标点
        coordinates = self.graph.get_path_coordinates(path)
        
        return {
            'path': path,
            'edges': edges,
            'total_distance': total_distance,
            'total_time': total_time,
            'coordinates': coordinates
        }
    
    def find_fastest_path(self, start_node_id: str, end_node_id: str) -> Dict[str, Any]:
        """
        查找最快路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            
        返回:
            包含路径信息的字典
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
            PathNotFoundError: 如果无法找到路径
        """
        return self.find_shortest_path(start_node_id, end_node_id, 'time')
    
    def find_path_with_waypoints(self, waypoints: List[str], 
                               weight_type: str = 'distance') -> Dict[str, Any]:
        """
        查找经过多个途经点的路径
        
        参数:
            waypoints: 途经点ID列表 (包括起点和终点)
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            包含路径信息的字典
            
        异常:
            NodeNotFoundError: 如果任何途经点不存在
            PathNotFoundError: 如果无法找到路径
        """
        if len(waypoints) < 2:
            raise ValueError("至少需要提供起点和终点")
        
        # 检查所有途经点是否存在
        for node_id in waypoints:
            if not self.graph.has_node(node_id):
                raise NodeNotFoundError(node_id)
        
        # 如果只有两个点，直接调用find_shortest_path
        if len(waypoints) == 2:
            return self.find_shortest_path(waypoints[0], waypoints[1], weight_type)
        
        # 分段查找路径
        path_segments = []
        edges_segments = []
        total_distance = 0.0
        total_time = 0.0
        coordinates_segments = []
        
        for i in range(len(waypoints) - 1):
            start = waypoints[i]
            end = waypoints[i + 1]
            
            segment_result = self.find_shortest_path(start, end, weight_type)
            
            # 如果不是第一段，去掉重复的起点
            if i > 0:
                path_segments.extend(segment_result['path'][1:])
            else:
                path_segments.extend(segment_result['path'])
            
            edges_segments.extend(segment_result['edges'])
            total_distance += segment_result['total_distance']
            total_time += segment_result['total_time']
            
            # 如果不是第一段，去掉重复的起点坐标
            if i > 0:
                coordinates_segments.extend(segment_result['coordinates'][1:])
            else:
                coordinates_segments.extend(segment_result['coordinates'])
        
        return {
            'path': path_segments,
            'edges': edges_segments,
            'total_distance': total_distance,
            'total_time': total_time,
            'coordinates': coordinates_segments
        }
    
    def find_alternative_paths(self, start_node_id: str, end_node_id: str, 
                              num_alternatives: int = 3, 
                              weight_type: str = 'distance') -> List[Dict[str, Any]]:
        """
        查找多条备选路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            num_alternatives: 备选路径数量
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            路径信息字典列表
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
        """
        # 检查节点是否存在
        if not self.graph.has_node(start_node_id):
            raise NodeNotFoundError(start_node_id)
        
        if not self.graph.has_node(end_node_id):
            raise NodeNotFoundError(end_node_id)
        
        # 如果起点和终点相同，直接返回
        if start_node_id == end_node_id:
            start_node = self.graph.get_node(start_node_id)
            return [{
                'path': [start_node_id],
                'edges': [],
                'total_distance': 0.0,
                'total_time': 0.0,
                'coordinates': [start_node.coordinate]
            }]
        
        # 使用Yen's K-最短路径算法查找备选路径
        paths = self._k_shortest_paths(start_node_id, end_node_id, num_alternatives, weight_type)
        
        results = []
        for path in paths:
            try:
                # 获取路径上的边
                edges = self.graph.get_path_edges(path)
                
                # 计算路径总长度和时间
                total_distance = sum(edge.length for edge in edges)
                total_time = sum(edge.get_travel_time() for edge in edges)
                
                # 获取路径上的坐标点
                coordinates = self.graph.get_path_coordinates(path)
                
                results.append({
                    'path': path,
                    'edges': edges,
                    'total_distance': total_distance,
                    'total_time': total_time,
                    'coordinates': coordinates
                })
            except Exception as e:
                # 如果处理某条路径时出错，跳过该路径
                continue
        
        return results
    
    def find_path_avoiding_areas(self, start_node_id: str, end_node_id: str, 
                               avoid_areas: List[List[Coordinate]], 
                               weight_type: str = 'distance') -> Dict[str, Any]:
        """
        查找避开特定区域的路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            avoid_areas: 要避开的区域列表，每个区域是一个坐标点列表，表示多边形
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            包含路径信息的字典
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
            PathNotFoundError: 如果无法找到路径
        """
        # 检查节点是否存在
        if not self.graph.has_node(start_node_id):
            raise NodeNotFoundError(start_node_id)
        
        if not self.graph.has_node(end_node_id):
            raise NodeNotFoundError(end_node_id)
        
        # 如果起点和终点相同，直接返回
        if start_node_id == end_node_id:
            start_node = self.graph.get_node(start_node_id)
            return {
                'path': [start_node_id],
                'edges': [],
                'total_distance': 0.0,
                'total_time': 0.0,
                'coordinates': [start_node.coordinate]
            }
        
        # 找出在避开区域内的节点
        from ..utils.geometry import is_point_in_polygon
        
        avoid_nodes = set()
        for node_id, node in self.graph.nodes.items():
            for area in avoid_areas:
                if is_point_in_polygon(node.coordinate, area):
                    avoid_nodes.add(node_id)
                    break
        
        # 使用修改后的Dijkstra算法查找路径，避开特定节点
        path = self._dijkstra_with_avoid(start_node_id, end_node_id, avoid_nodes, weight_type)
        
        # 获取路径上的边
        edges = self.graph.get_path_edges(path)
        
        # 计算路径总长度和时间
        total_distance = sum(edge.length for edge in edges)
        total_time = sum(edge.get_travel_time() for edge in edges)
        
        # 获取路径上的坐标点
        coordinates = self.graph.get_path_coordinates(path)
        
        return {
            'path': path,
            'edges': edges,
            'total_distance': total_distance,
            'total_time': total_time,
            'coordinates': coordinates
        }
    
    def find_path_with_constraints(self, start_node_id: str, end_node_id: str, 
                                 constraints: Dict[str, Any], 
                                 weight_type: str = 'distance') -> Dict[str, Any]:
        """
        查找满足特定约束条件的路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            constraints: 约束条件字典，可包含以下键:
                - max_distance: 最大距离 (米)
                - max_time: 最大时间 (小时)
                - avoid_road_types: 要避开的道路类型列表
                - prefer_road_types: 优先选择的道路类型列表
                - avoid_node_ids: 要避开的节点ID列表
                - must_visit_node_ids: 必须经过的节点ID列表
            weight_type: 权重类型 ('distance', 'time', 'custom')
            
        返回:
            包含路径信息的字典
            
        异常:
            NodeNotFoundError: 如果起点或终点不存在
            PathNotFoundError: 如果无法找到路径
        """
        # 检查节点是否存在
        if not self.graph.has_node(start_node_id):
            raise NodeNotFoundError(start_node_id)
        
        if not self.graph.has_node(end_node_id):
            raise NodeNotFoundError(end_node_id)
        
        # 如果起点和终点相同，直接返回
        if start_node_id == end_node_id:
            start_node = self.graph.get_node(start_node_id)
            return {
                'path': [start_node_id],
                'edges': [],
                'total_distance': 0.0,
                'total_time': 0.0,
                'coordinates': [start_node.coordinate]
            }
        
        # 处理必须经过的节点
        must_visit = constraints.get('must_visit_node_ids', [])
        if must_visit:
            # 添加起点和终点
            waypoints = [start_node_id] + must_visit + [end_node_id]
            return self.find_path_with_waypoints(waypoints, weight_type)
        
        # 处理要避开的节点
        avoid_nodes = set(constraints.get('avoid_node_ids', []))
        
        # 处理要避开的道路类型
        avoid_road_types = set(constraints.get('avoid_road_types', []))
        
        # 处理优先选择的道路类型
        prefer_road_types = set(constraints.get('prefer_road_types', []))
        
        # 自定义边权重函数
        def edge_weight_func(start_id: str, end_id: str) -> float:
            edge_id = self.graph.adjacency[start_id][end_id]
            edge = self.graph.edges[edge_id]
            
            # 基础权重
            if weight_type == 'distance':
                weight = edge.length
            elif weight_type == 'time':
                weight = edge.get_travel_time() * 3600  # 转换为秒
            else:  # 'custom'
                weight = edge.length + edge.get_travel_time() * 3600 * 0.1
            
            # 如果是要避开的道路类型，增加权重
            if edge.road_type in avoid_road_types:
                weight *= 10.0
            
            # 如果是优先选择的道路类型，减少权重
            if edge.road_type in prefer_road_types:
                weight *= 0.5
            
            return weight
        
        # 使用修改后的Dijkstra算法查找路径
        path = self._dijkstra_with_custom_weight(start_node_id, end_node_id, avoid_nodes, edge_weight_func)
        
        # 获取路径上的边
        edges = self.graph.get_path_edges(path)
        
        # 计算路径总长度和时间
        total_distance = sum(edge.length for edge in edges)
        total_time = sum(edge.get_travel_time() for edge in edges)
        
        # 检查是否满足最大距离和时间约束
        max_distance = constraints.get('max_distance')
        if max_distance is not None and total_distance > max_distance:
            raise PathNotFoundError(start_node_id, end_node_id, 
                                   f"无法找到满足最大距离约束的路径 (当前: {total_distance:.1f}m, 最大: {max_distance:.1f}m)")
        
        max_time = constraints.get('max_time')
        if max_time is not None and total_time > max_time:
            raise PathNotFoundError(start_node_id, end_node_id, 
                                   f"无法找到满足最大时间约束的路径 (当前: {total_time:.2f}h, 最大: {max_time:.2f}h)")
        
        # 获取路径上的坐标点
        coordinates = self.graph.get_path_coordinates(path)
        
        return {
            'path': path,
            'edges': edges,
            'total_distance': total_distance,
            'total_time': total_time,
            'coordinates': coordinates
        }
    
    def _dijkstra(self, start_node_id: str, end_node_id: str, 
                 weight_type: str = 'distance', weight_func=None) -> List[str]:
        """
        使用Dijkstra算法查找最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型
            weight_func: 自定义权重函数 (start_node_id, end_node_id) -> weight
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 初始化
        distances = {node_id: float('inf') for node_id in self.graph.nodes}
        distances[start_node_id] = 0
        previous = {node_id: None for node_id in self.graph.nodes}
        unvisited = set(self.graph.nodes.keys())
        
        while unvisited:
            # 找到距离最小的未访问节点
            current = min(unvisited, key=lambda node_id: distances[node_id])
            
            # 如果当前节点是终点，或者当前节点的距离是无穷大（无法到达），则结束
            if current == end_node_id or distances[current] == float('inf'):
                break
            
            # 移除当前节点
            unvisited.remove(current)
            
            # 更新邻居节点的距离
            for neighbor, edge_id in self.graph.adjacency[current].items():
                if neighbor in unvisited:
                    # 优先使用自定义权重函数
                    if weight_func is not None:
                        weight = weight_func(current, neighbor)
                    else:
                        weight = self.graph.get_edge_weight(current, neighbor, weight_type)
                    
                    new_distance = distances[current] + weight
                    
                    if new_distance < distances[neighbor]:
                        distances[neighbor] = new_distance
                        previous[neighbor] = current
        
        # 如果终点不可达
        if distances[end_node_id] == float('inf'):
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _a_star(self, start_node_id: str, end_node_id: str, weight_type: str = 'distance') -> List[str]:
        """
        使用A*算法查找最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            weight_type: 权重类型
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 启发式函数：估计从当前节点到终点的距离
        def heuristic(node_id):
            # 使用直线距离作为启发式函数
            node = self.graph.nodes[node_id]
            end_node = self.graph.nodes[end_node_id]
            return haversine_distance(node.coordinate, end_node.coordinate)
        
        # 初始化
        open_set = {start_node_id}
        closed_set = set()
        
        g_score = {node_id: float('inf') for node_id in self.graph.nodes}
        g_score[start_node_id] = 0
        
        f_score = {node_id: float('inf') for node_id in self.graph.nodes}
        f_score[start_node_id] = heuristic(start_node_id)
        
        previous = {node_id: None for node_id in self.graph.nodes}
        
        while open_set:
            # 找到f_score最小的节点
            current = min(open_set, key=lambda node_id: f_score[node_id])
            
            # 如果当前节点是终点，则结束
            if current == end_node_id:
                break
            
            # 移动当前节点
            open_set.remove(current)
            closed_set.add(current)
            
            # 更新邻居节点
            for neighbor, edge_id in self.graph.adjacency[current].items():
                if neighbor in closed_set:
                    continue
                
                weight = self.graph.get_edge_weight(current, neighbor, weight_type)
                tentative_g_score = g_score[current] + weight
                
                if neighbor not in open_set:
                    open_set.add(neighbor)
                elif tentative_g_score >= g_score[neighbor]:
                    continue
                
                previous[neighbor] = current
                g_score[neighbor] = tentative_g_score
                f_score[neighbor] = g_score[neighbor] + heuristic(neighbor)
        
        # 如果终点不可达
        if previous[end_node_id] is None and start_node_id != end_node_id:
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _dijkstra_with_avoid(self, start_node_id: str, end_node_id: str, 
                           avoid_nodes: Set[str], weight_type: str = 'distance') -> List[str]:
        """
        使用修改后的Dijkstra算法查找路径，避开特定节点
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            avoid_nodes: 要避开的节点ID集合
            weight_type: 权重类型
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 如果起点或终点在避开节点中，则无法找到路径
        if start_node_id in avoid_nodes or end_node_id in avoid_nodes:
            raise PathNotFoundError(start_node_id, end_node_id, "起点或终点在避开区域内")
        
        # 初始化
        distances = {node_id: float('inf') for node_id in self.graph.nodes if node_id not in avoid_nodes}
        distances[start_node_id] = 0
        previous = {node_id: None for node_id in self.graph.nodes if node_id not in avoid_nodes}
        unvisited = set(node_id for node_id in self.graph.nodes.keys() if node_id not in avoid_nodes)
        
        while unvisited:
            # 找到距离最小的未访问节点
            current = min(unvisited, key=lambda node_id: distances[node_id])
            
            # 如果当前节点是终点，或者当前节点的距离是无穷大（无法到达），则结束
            if current == end_node_id or distances[current] == float('inf'):
                break
            
            # 移除当前节点
            unvisited.remove(current)
            
            # 更新邻居节点的距离
            for neighbor, edge_id in self.graph.adjacency[current].items():
                if neighbor in unvisited:
                    weight = self.graph.get_edge_weight(current, neighbor, weight_type)
                    new_distance = distances[current] + weight
                    
                    if new_distance < distances.get(neighbor, float('inf')):
                        distances[neighbor] = new_distance
                        previous[neighbor] = current
        
        # 如果终点不可达
        if end_node_id not in distances or distances[end_node_id] == float('inf'):
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _dijkstra_with_custom_weight(self, start_node_id: str, end_node_id: str, 
                                   avoid_nodes: Set[str], 
                                   weight_func: Callable[[str, str], float]) -> List[str]:
        """
        使用修改后的Dijkstra算法查找路径，使用自定义权重函数
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            avoid_nodes: 要避开的节点ID集合
            weight_func: 自定义权重函数，接受起点和终点ID，返回权重
            
        返回:
            路径上的节点ID列表
            
        异常:
            PathNotFoundError: 如果无法找到路径
        """
        # 如果起点或终点在避开节点中，则无法找到路径
        if start_node_id in avoid_nodes or end_node_id in avoid_nodes:
            raise PathNotFoundError(start_node_id, end_node_id, "起点或终点在避开区域内")
        
        # 初始化
        distances = {node_id: float('inf') for node_id in self.graph.nodes if node_id not in avoid_nodes}
        distances[start_node_id] = 0
        previous = {node_id: None for node_id in self.graph.nodes if node_id not in avoid_nodes}
        unvisited = set(node_id for node_id in self.graph.nodes.keys() if node_id not in avoid_nodes)
        
        while unvisited:
            # 找到距离最小的未访问节点
            current = min(unvisited, key=lambda node_id: distances[node_id])
            
            # 如果当前节点是终点，或者当前节点的距离是无穷大（无法到达），则结束
            if current == end_node_id or distances[current] == float('inf'):
                break
            
            # 移除当前节点
            unvisited.remove(current)
            
            # 更新邻居节点的距离
            for neighbor, edge_id in self.graph.adjacency[current].items():
                if neighbor in unvisited:
                    weight = weight_func(current, neighbor)
                    new_distance = distances[current] + weight
                    
                    if new_distance < distances.get(neighbor, float('inf')):
                        distances[neighbor] = new_distance
                        previous[neighbor] = current
        
        # 如果终点不可达
        if end_node_id not in distances or distances[end_node_id] == float('inf'):
            raise PathNotFoundError(start_node_id, end_node_id)
        
        # 构建路径
        path = []
        current = end_node_id
        
        while current:
            path.append(current)
            current = previous[current]
        
        # 反转路径，使其从起点到终点
        return path[::-1]
    
    def _k_shortest_paths(self, start_node_id: str, end_node_id: str, 
                        k: int, weight_type: str = 'distance') -> List[List[str]]:
        """
        使用Yen's K-最短路径算法查找k条最短路径
        
        参数:
            start_node_id: 起始节点ID
            end_node_id: 终止节点ID
            k: 路径数量
            weight_type: 权重类型
            
        返回:
            路径列表，每个路径是节点ID列表
        """
        # 找到第一条最短路径
        try:
            shortest_path = self._dijkstra(start_node_id, end_node_id, weight_type)
        except PathNotFoundError:
            return []
        
        # 初始化结果列表和候选路径优先队列
        A = [shortest_path]
        B = []
        
        # 如果只需要一条路径，直接返回
        if k <= 1:
            return A
        
        for i in range(1, k):
            # 对上一条路径的每个节点，尝试找到备选路径
            prev_path = A[i-1]
            
            for j in range(len(prev_path) - 1):
                # 分叉节点
                spur_node = prev_path[j]
                # 从起点到分叉节点的路径
                root_path = prev_path[:j+1]
                
                # 临时移除已有路径中的边，以便找到新的路径
                removed_edges = []
                
                for path in A:
                    if len(path) > j + 1 and path[:j+1] == root_path:
                        u = path[j]
                        v = path[j+1]
                        if self.graph.has_edge_between(u, v):
                            edge_id = self.graph.adjacency[u][v]
                            removed_edges.append((u, v, edge_id))
                            self.graph.remove_edge(edge_id)
                
                # 临时移除根路径中的节点，以避免环路
                removed_nodes = []
                for node in root_path[:-1]:  # 不包括分叉节点
                    if node != spur_node and self.graph.has_node(node):
                        removed_nodes.append(node)
                        self.graph.remove_node(node)
                
                # 从分叉节点到终点查找路径
                try:
                    spur_path = self._dijkstra(spur_node, end_node_id, weight_type)
                    # 组合根路径和分叉路径
                    candidate_path = root_path[:-1] + spur_path
                    
                    # 检查是否已经存在
                    if candidate_path not in A and candidate_path not in [p for _, p in B]:
                        # 计算路径长度
                        path_length = 0.0
                        for i in range(len(candidate_path) - 1):
                            u = candidate_path[i]
                            v = candidate_path[i+1]
                            if self.graph.has_edge_between(u, v):
                                path_length += self.graph.get_edge_weight(u, v, weight_type)
                        
                        heapq.heappush(B, (path_length, candidate_path))
                
                except PathNotFoundError:
                    pass
                
                # 恢复移除的边
                for u, v, edge_id in removed_edges:
                    edge = self.graph.edges[edge_id]
                    self.graph.add_edge(edge)
                
                # 恢复移除的节点
                for node_id in removed_nodes:
                    node = self.graph.nodes[node_id]
                    self.graph.add_node(node)
            
            # 如果没有更多候选路径，结束
            if not B:
                break
            
            # 添加最短的候选路径到结果中
            _, next_path = heapq.heappop(B)
            A.append(next_path)
        
        return A


"""
观察者模块 - 提供地理空间环境中的观察者实现
"""
from typing import List, Dict, Any, Optional, Set, Tuple
import math
import time

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..utils.distance import haversine_distance, bearing
from .exceptions import ObserverError


class Observer:
    """
    观察者类，代表在地理空间环境中的一个智能体，
    能够感知周围的环境并与地理空间图交互
    """
    
    def __init__(self, location: Coordinate, heading: float = 0.0, 
                 perception_range: float = 100.0, fov: float = 120.0):
        """
        初始化观察者
        
        参数:
            location: 观察者的当前位置坐标
            heading: 观察者的朝向 (度，北为0度，顺时针增加)
            perception_range: 感知范围 (米)
            fov: 视野角度 (度)
        """
        self.location = location
        self.heading = heading
        self.perception_range = perception_range
        self.fov = fov
        self.speed = 0.0  # 当前速度 (米/秒)
        self.last_update_time = time.time()
        self.path_history: List[Coordinate] = [location]  # 历史轨迹
        self.matched_road: Optional[Dict[str, Any]] = None  # 当前匹配的道路信息
        self.visible_pois: List[POI] = []  # 当前可见的POI
        self.visible_roads: List[Road] = []  # 当前可见的道路
    
    def update_location(self, new_location: Coordinate, new_heading: Optional[float] = None) -> None:
        """
        更新观察者的位置和朝向
        
        参数:
            new_location: 新的位置坐标
            new_heading: 新的朝向 (度)，如果为None则根据移动方向计算
        """
        # 计算移动距离
        distance = haversine_distance(self.location, new_location)
        
        # 计算时间差
        current_time = time.time()
        time_diff = current_time - self.last_update_time
        
        # 更新速度 (如果时间差大于0)
        if time_diff > 0:
            self.speed = distance / time_diff
        
        # 如果没有提供新的朝向，则根据移动方向计算
        if new_heading is None and distance > 0.1:  # 只有当移动距离足够大时才更新朝向
            self.heading = bearing(self.location, new_location)
        elif new_heading is not None:
            self.heading = new_heading
        
        # 更新位置和时间
        self.location = new_location
        self.last_update_time = current_time
        
        # 记录历史轨迹
        self.path_history.append(new_location)
    
    def is_poi_visible(self, poi: POI) -> bool:
        """
        判断POI是否在观察者的视野范围内
        
        参数:
            poi: 要检查的POI
            
        返回:
            如果POI在视野范围内则返回True，否则返回False
        """
        # 计算距离
        distance = haversine_distance(self.location, poi.coordinate)
        
        # 调试信息
        print(f"Checking POI visibility: {poi.poi_id}")
        print(f"Distance: {distance} (range: {self.perception_range})")
        
        # 如果距离超出感知范围，则不可见
        if distance > self.perception_range:
            print("POI too far")
            return False
        
        # 如果距离为0（自身位置），则可见
        if distance == 0:
            print("POI at same location")
            return True
        
        # 如果视野角度为360度，则可见
        if self.fov >= 360:
            print("360 FOV - always visible")
            return True
        
        # 计算POI相对于观察者的方位角
        poi_bearing = bearing(self.location, poi.coordinate)
        print(f"POI bearing: {poi_bearing} (heading: {self.heading})")
        
        # 计算方位角差值
        angle_diff = abs((poi_bearing - self.heading + 180) % 360 - 180)
        print(f"Angle difference: {angle_diff} (FOV/2: {self.fov/2})")
        
        # 如果在视野角度内，则可见
        visible = angle_diff <= self.fov / 2
        print(f"Visible: {visible}")
        return visible
    
    def is_road_visible(self, road: Road) -> bool:
        """
        判断道路是否在观察者的视野范围内
        
        参数:
            road: 要检查的道路
            
        返回:
            如果道路在视野范围内则返回True，否则返回False
        """
        # 检查道路的每个线段
        for i in range(len(road.coordinates) - 1):
            start = road.coordinates[i]
            end = road.coordinates[i + 1]
            
            # 计算线段中点
            mid_lat = (start.latitude + end.latitude) / 2
            mid_lon = (start.longitude + end.longitude) / 2
            mid_elev = None
            if start.elevation is not None and end.elevation is not None:
                mid_elev = (start.elevation + end.elevation) / 2
            mid_point = Coordinate(mid_lat, mid_lon, mid_elev)
            
            # 计算到中点的距离
            distance = haversine_distance(self.location, mid_point)
            
            # 如果距离在感知范围内
            if distance <= self.perception_range:
                # 如果视野角度为360度，则可见
                if self.fov >= 360:
                    return True
                
                # 计算中点相对于观察者的方位角
                mid_bearing = bearing(self.location, mid_point)
                
                # 计算方位角差值
                angle_diff = abs((mid_bearing - self.heading + 180) % 360 - 180)
                
                # 如果在视野角度内，则可见
                if angle_diff <= self.fov / 2:
                    return True
        
        return False
    
    def update_perception(self, pois: List[POI], roads: List[Road]) -> Dict[str, Any]:
        """
        更新观察者对周围环境的感知
        
        参数:
            pois: 环境中的所有POI
            roads: 环境中的所有道路
            
        返回:
            包含感知结果的字典
        """
        # 更新可见的POI
        self.visible_pois = [poi for poi in pois if self.is_poi_visible(poi)]
        
        # 更新可见的道路
        self.visible_roads = [road for road in roads if self.is_road_visible(road)]
        
        # 返回感知结果
        return {
            'visible_pois': self.visible_pois,
            'visible_roads': self.visible_roads,
            'location': self.location,
            'heading': self.heading,
            'speed': self.speed,
            'matched_road': self.matched_road
        }
    
    def set_matched_road(self, matched_road: Optional[Dict[str, Any]]) -> None:
        """
        设置观察者当前匹配的道路
        
        参数:
            matched_road: 匹配的道路信息，如果为None表示未匹配到道路
        """
        self.matched_road = matched_road
    
    def get_path_history(self, max_points: Optional[int] = None) -> List[Coordinate]:
        """
        获取观察者的历史轨迹
        
        参数:
            max_points: 最大返回点数，如果为None则返回所有点
            
        返回:
            历史轨迹坐标列表
        """
        if max_points is None or max_points >= len(self.path_history):
            return self.path_history.copy()
        
        # 返回最近的max_points个点
        return self.path_history[-max_points:]
    
    def clear_path_history(self) -> None:
        """清空历史轨迹，但保留当前位置"""
        current_location = self.path_history[-1] if self.path_history else self.location
        self.path_history = [current_location]
    
    def get_nearby_pois(self, max_distance: Optional[float] = None) -> List[Tuple[POI, float]]:
        """
        获取观察者附近的POI及其距离
        
        参数:
            max_distance: 最大距离 (米)，如果为None则使用感知范围
            
        返回:
            (POI, 距离) 元组的列表，按距离升序排序
        """
        if max_distance is None:
            max_distance = self.perception_range
        
        result = []
        for poi in self.visible_pois:
            distance = haversine_distance(self.location, poi.coordinate)
            if distance <= max_distance:
                result.append((poi, distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[1])
    
    def get_nearby_roads(self, max_distance: Optional[float] = None) -> List[Tuple[Road, float]]:
        """
        获取观察者附近的道路及其距离
        
        参数:
            max_distance: 最大距离 (米)，如果为None则使用感知范围
            
        返回:
            (Road, 距离) 元组的列表，按距离升序排序
        """
        if max_distance is None:
            max_distance = self.perception_range
        
        result = []
        for road in self.visible_roads:
            # 计算到道路的最小距离
            min_distance = float('inf')
            for i in range(len(road.coordinates) - 1):
                start = road.coordinates[i]
                end = road.coordinates[i + 1]
                
                # 使用点到线段的距离
                from ..utils.geometry import point_to_line_distance
                distance = point_to_line_distance(self.location, start, end)
                min_distance = min(min_distance, distance)
            
            if min_distance <= max_distance:
                result.append((road, min_distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[1])
    
    def __str__(self) -> str:
        """返回人类可读的观察者字符串"""
        return f"Observer at {self.location}, heading: {self.heading:.1f}°, speed: {self.speed:.1f} m/s"


class MultiObserverManager:
    """
    多观察者管理器，用于管理和更新多个观察者
    """
    
    def __init__(self):
        """初始化多观察者管理器"""
        self.observers: Dict[str, Observer] = {}
    
    def add_observer(self, observer_id: str, observer: Observer) -> None:
        """
        添加观察者
        
        参数:
            observer_id: 观察者ID
            observer: 观察者对象
        """
        if observer_id in self.observers:
            raise ObserverError(f"观察者ID已存在: {observer_id}")
        
        self.observers[observer_id] = observer
    
    def remove_observer(self, observer_id: str) -> bool:
        """
        移除观察者
        
        参数:
            observer_id: 观察者ID
            
        返回:
            如果成功移除则返回True，否则返回False
        """
        if observer_id in self.observers:
            del self.observers[observer_id]
            return True
        return False
    
    def get_observer(self, observer_id: str) -> Observer:
        """
        获取观察者
        
        参数:
            observer_id: 观察者ID
            
        返回:
            观察者对象
            
        异常:
            ObserverError: 如果观察者不存在
        """
        if observer_id not in self.observers:
            raise ObserverError(f"观察者不存在: {observer_id}")
        
        return self.observers[observer_id]
    
    def update_all_perceptions(self, pois: List[POI], roads: List[Road]) -> Dict[str, Dict[str, Any]]:
        """
        更新所有观察者的感知
        
        参数:
            pois: 环境中的所有POI
            roads: 环境中的所有道路
            
        返回:
            观察者ID到感知结果的映射
        """
        results = {}
        for observer_id, observer in self.observers.items():
            results[observer_id] = observer.update_perception(pois, roads)
        
        return results
    
    def get_observers_in_range(self, location: Coordinate, max_distance: float) -> List[Tuple[str, Observer, float]]:
        """
        获取指定位置附近的观察者
        
        参数:
            location: 位置坐标
            max_distance: 最大距离 (米)
            
        返回:
            (观察者ID, 观察者对象, 距离) 元组的列表，按距离升序排序
        """
        result = []
        for observer_id, observer in self.observers.items():
            distance = haversine_distance(location, observer.location)
            if distance <= max_distance:
                result.append((observer_id, observer, distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[2])
    
    def __len__(self) -> int:
        """返回观察者数量"""
        return len(self.observers)
    
    def __contains__(self, observer_id: str) -> bool:
        """检查观察者ID是否存在"""
        return observer_id in self.observers
    
    def __iter__(self):
        """迭代所有观察者ID"""
        return iter(self.observers)
    
    def items(self):
        """迭代所有(观察者ID, 观察者对象)对"""
        return self.observers.items()


"""
道路模块 - 提供道路的定义和操作
"""
from dataclasses import dataclass, field
from typing import Dict, Any, List, Optional, Tuple
from uuid import uuid4

from .coordinate import Coordinate


@dataclass
class Road:
    """
    道路类，表示地图上的道路或路段
    
    属性:
        road_id: 道路的唯一标识符
        name: 道路名称
        start_node_id: 起始节点ID
        end_node_id: 终止节点ID
        coordinates: 道路的坐标点列表，表示道路的形状
        road_type: 道路类型 (如主干道、次干道、街道等)
        properties: 道路的其他属性
        length: 道路长度 (米)
        speed_limit: 速度限制 (km/h)
        bidirectional: 是否为双向道路
    """
    start_node_id: str  # 格式应为 node_{经度}_{纬度}
    end_node_id: str    # 格式应为 node_{经度}_{纬度}
    coordinates: List[Coordinate]
    road_id: str = field(default_factory=lambda: str(uuid4()))
    name: str = ""
    road_type: str = "street"
    properties: Dict[str, Any] = field(default_factory=dict)
    length: Optional[float] = None
    speed_limit: float = 50.0  # 默认速度限制为50km/h
    bidirectional: bool = True
    
    def __post_init__(self):
        """初始化后的处理"""
        # 确保name不为None
        if self.name is None:
            self.name = ""
        
        # 确保road_type不为None
        if self.road_type is None:
            self.road_type = "street"
        
        # 确保properties不为None
        if self.properties is None:
            self.properties = {}
        
        # 确保coordinates不为None且至少有两个点
        if self.coordinates is None or len(self.coordinates) < 2:
            raise ValueError("道路必须至少包含两个坐标点")
        
        # 如果未提供长度，则计算长度
        if self.length is None:
            self.length = self._calculate_length()
    
    def _calculate_length(self) -> float:
        """
        计算道路长度 (米)
        
        返回:
            道路长度 (米)
        """
        from ..utils.distance import haversine_distance
        
        total_length = 0.0
        for i in range(len(self.coordinates) - 1):
            total_length += haversine_distance(
                self.coordinates[i], self.coordinates[i + 1]
            )
        
        return total_length
    
    def get_start_coordinate(self) -> Coordinate:
        """
        获取道路的起始坐标
        
        返回:
            起始坐标
        """
        return self.coordinates[0]
    
    def get_end_coordinate(self) -> Coordinate:
        """
        获取道路的终止坐标
        
        返回:
            终止坐标
        """
        return self.coordinates[-1]
    
    def get_node_ids(self) -> Tuple[str, str]:
        """
        获取道路连接的节点ID
        
        返回:
            (起始节点ID, 终止节点ID)
        """
        return (self.start_node_id, self.end_node_id)
    
    def get_travel_time(self) -> float:
        """
        计算道路的理论通行时间 (小时)
        
        返回:
            通行时间 (小时)
        """
        # 长度单位为米，速度单位为km/h，需要进行单位转换
        return self.length / 1000 / self.speed_limit
    
    def get_property(self, key: str, default: Any = None) -> Any:
        """
        获取道路的属性值
        
        参数:
            key: 属性键
            default: 如果属性不存在，返回的默认值
            
        返回:
            属性值或默认值
        """
        return self.properties.get(key, default)
    
    def set_property(self, key: str, value: Any) -> None:
        """
        设置道路的属性值
        
        参数:
            key: 属性键
            value: 属性值
        """
        self.properties[key] = value
    
    def __str__(self) -> str:
        """返回人类可读的道路字符串"""
        if self.name:
            return f"{self.name} ({self.road_type}, {self.length:.1f}m)"
        return f"{self.road_type} road ({self.length:.1f}m)"
    
    def __eq__(self, other) -> bool:
        """判断两条道路是否相等"""
        if not isinstance(other, Road):
            return False
        return self.road_id == other.road_id
    
    def __hash__(self) -> int:
        """计算道路的哈希值，使其可以用作字典键"""
        return hash(self.road_id)


def create_road_from_dict(data: Dict[str, Any]) -> Road:
    """
    从字典创建Road对象
    
    参数:
        data: 包含道路数据的字典
        
    返回:
        Road对象
    """
    # 提取必要的节点ID
    start_node_id = data.get('start_node_id')
    end_node_id = data.get('end_node_id')
    
    if not start_node_id or not end_node_id:
        raise ValueError("道路数据缺少起始或终止节点ID")
    
    # 提取坐标列表
    coordinates = []
    if 'coordinates' in data and isinstance(data['coordinates'], list):
        for coord_data in data['coordinates']:
            if isinstance(coord_data, dict):
                lat = coord_data.get('latitude')
                lon = coord_data.get('longitude')
                elev = coord_data.get('elevation')
                
                if lat is not None and lon is not None:
                    coordinates.append(Coordinate(lat, lon, elev))
            elif isinstance(coord_data, (list, tuple)) and len(coord_data) >= 2:
                lat, lon = coord_data[0], coord_data[1]
                elev = coord_data[2] if len(coord_data) > 2 else None
                coordinates.append(Coordinate(lat, lon, elev))
    
    if len(coordinates) < 2:
        raise ValueError("道路必须至少包含两个坐标点")
    
    # 提取其他道路属性
    road_id = data.get('id') or data.get('road_id', str(uuid4()))
    name = data.get('name', "")
    road_type = data.get('type') or data.get('road_type', "street")
    length = data.get('length')
    speed_limit = data.get('speed_limit', 50.0)
    bidirectional = data.get('bidirectional', True)
    
    # 处理properties字段
    properties = {}
    if 'properties' in data and isinstance(data['properties'], dict):
        properties = data['properties']
    else:
        # 提取属性字典，排除已处理的键
        properties = {k: v for k, v in data.items() 
                     if k not in ('id', 'road_id', 'name', 'type', 'road_type', 
                                 'start_node_id', 'end_node_id', 'coordinates',
                                 'length', 'speed_limit', 'bidirectional',
                                 'properties')}
    
    # 创建Road对象
    road = Road(
        road_id=road_id,
        name=name,
        start_node_id=start_node_id,
        end_node_id=end_node_id,
        coordinates=coordinates,
        road_type=road_type,
        properties=properties,
        length=length,
        speed_limit=speed_limit,
        bidirectional=bidirectional
    )
    
    return road


"""
地理感知模块 - 提供地理空间环境感知功能
"""
from typing import List, Dict, Any, Tuple, Set, Optional
import math
from collections import defaultdict

from ..entities.coordinate import Coordinate
from ..entities.poi import POI
from ..entities.road import Road
from ..core.graph import LocationGraph
from ..core.observer import Observer
from ..utils.distance import haversine_distance, bearing
from ..utils.geometry import point_to_polyline_distance


class PerceptionEngine:
    """
    地理感知引擎，提供地理空间环境感知功能
    """
    
    def __init__(self, graph: LocationGraph):
        """
        初始化地理感知引擎
        
        参数:
            graph: 地理空间图
        """
        self.graph = graph
    
    def detect_nearby_pois(self, location: Coordinate, radius: float, 
                          poi_types: Optional[List[str]] = None) -> List[Tuple[POI, float]]:
        """
        检测指定位置附近的POI
        
        参数:
            location: 位置坐标
            radius: 搜索半径 (米)
            poi_types: POI类型列表，如果为None则检测所有类型
            
        返回:
            (POI, 距离) 元组的列表，按距离升序排序
        """
        result = []
        
        for poi in self.graph.nodes.values():
            # 如果指定了POI类型，则只检测指定类型
            if poi_types and poi.poi_type not in poi_types:
                continue
            
            distance = haversine_distance(location, poi.coordinate)
            if distance <= radius:
                result.append((poi, distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[1])
    
    def detect_nearby_roads(self, location: Coordinate, radius: float, 
                           road_types: Optional[List[str]] = None) -> List[Tuple[Road, float]]:
        """
        检测指定位置附近的道路
        
        参数:
            location: 位置坐标
            radius: 搜索半径 (米)
            road_types: 道路类型列表，如果为None则检测所有类型
            
        返回:
            (Road, 距离) 元组的列表，按距离升序排序
        """
        result = []
        
        for road in self.graph.edges.values():
            # 如果指定了道路类型，则只检测指定类型
            if road_types and road.road_type not in road_types:
                continue
            
            # 计算点到道路的最短距离
            min_distance, _, _ = point_to_polyline_distance(location, road.coordinates)
            
            if min_distance <= radius:
                result.append((road, min_distance))
        
        # 按距离排序
        return sorted(result, key=lambda x: x[1])
    
    def match_coordinate(self, location: Coordinate, max_distance: float = 50.0) -> Dict[str, Any]:
        """
        将位置匹配到最近的道路上
        
        参数:
            location: 位置坐标
            max_distance: 最大匹配距离 (米)
            
        返回:
            包含匹配结果的字典，如果没有匹配到则返回None
        """
        return self.graph.match_coordinate(location)
    
    def match_trajectory(self, trajectory: List[Coordinate], max_gap: int = 5) -> List[Dict[str, Any]]:
        """
        将轨迹匹配到道路上
        
        参数:
            trajectory: 轨迹坐标列表
            max_gap: 允许的最大连续未匹配点数
            
        返回:
            匹配结果列表
        """
        return self.graph.match_coordinates(trajectory, max_gap)
    
    def detect_turn_events(self, trajectory: List[Coordinate], 
                          angle_threshold: float = 30.0) -> List[Dict[str, Any]]:
        """
        检测轨迹中的转弯事件
        
        参数:
            trajectory: 轨迹坐标列表
            angle_threshold: 转弯角度阈值 (度)
            
        返回:
            转弯事件列表，每个事件是一个字典:
            {
                'index': 转弯点在轨迹中的索引,
                'coordinate': 转弯点坐标,
                'angle': 转弯角度 (度),
                'direction': 转弯方向 ('left' 或 'right')
            }
        """
        if len(trajectory) < 3:
            return []
        
        turn_events = []
        
        for i in range(1, len(trajectory) - 1):
            prev_coord = trajectory[i - 1]
            curr_coord = trajectory[i]
            next_coord = trajectory[i + 1]
            
            # 计算前一段和后一段的方位角
            bearing1 = bearing(prev_coord, curr_coord)
            bearing2 = bearing(curr_coord, next_coord)
            
            # 计算转弯角度
            angle_diff = abs((bearing2 - bearing1 + 180) % 360 - 180)
            
            # 如果角度大于阈值，则认为是转弯
            if angle_diff > angle_threshold:
                # 判断转弯方向
                direction = 'right' if (bearing2 - bearing1 + 360) % 360 < 180 else 'left'
                
                turn_events.append({
                    'index': i,
                    'coordinate': curr_coord,
                    'angle': angle_diff,
                    'direction': direction
                })
        
        return turn_events
    
    def detect_stop_events(self, trajectory: List[Coordinate], timestamps: List[float], 
                         speed_threshold: float = 0.5, duration_threshold: float = 60.0) -> List[Dict[str, Any]]:
        """
        检测轨迹中的停止事件
        
        参数:
            trajectory: 轨迹坐标列表
            timestamps: 对应的时间戳列表 (秒)
            speed_threshold: 速度阈值 (米/秒)，低于此值视为停止
            duration_threshold: 持续时间阈值 (秒)，超过此值视为有效停止
            
        返回:
            停止事件列表，每个事件是一个字典:
            {
                'start_index': 停止开始点在轨迹中的索引,
                'end_index': 停止结束点在轨迹中的索引,
                'coordinate': 停止位置坐标 (取平均值),
                'duration': 停止持续时间 (秒),
                'nearby_pois': 附近的POI列表
            }
        """
        if len(trajectory) != len(timestamps):
            raise ValueError("轨迹坐标列表和时间戳列表长度不一致")
        
        if len(trajectory) < 2:
            return []
        
        stop_events = []
        current_stop = None
        
        for i in range(1, len(trajectory)):
            prev_coord = trajectory[i - 1]
            curr_coord = trajectory[i]
            
            # 计算距离和时间差
            distance = haversine_distance(prev_coord, curr_coord)
            time_diff = timestamps[i] - timestamps[i - 1]
            
            # 计算速度
            speed = distance / time_diff if time_diff > 0 else 0
            
            # 如果速度低于阈值，可能是停止
            if speed < speed_threshold:
                if current_stop is None:
                    # 开始新的停止事件
                    current_stop = {
                        'start_index': i - 1,
                        'coordinates': [prev_coord, curr_coord],
                        'timestamps': [timestamps[i - 1], timestamps[i]]
                    }
                else:
                    # 继续当前停止事件
                    current_stop['coordinates'].append(curr_coord)
                    current_stop['timestamps'].append(timestamps[i])
            else:
                # 如果当前有停止事件，检查其持续时间
                if current_stop is not None:
                    duration = current_stop['timestamps'][-1] - current_stop['timestamps'][0]
                    
                    if duration >= duration_threshold:
                        # 计算停止位置的平均坐标
                        avg_lat = sum(c.latitude for c in current_stop['coordinates']) / len(current_stop['coordinates'])
                        avg_lon = sum(c.longitude for c in current_stop['coordinates']) / len(current_stop['coordinates'])
                        avg_coord = Coordinate(avg_lat, avg_lon)
                        
                        # 查找附近的POI
                        nearby_pois = self.detect_nearby_pois(avg_coord, 100.0)
                        
                        stop_events.append({
                            'start_index': current_stop['start_index'],
                            'end_index': i - 1,
                            'coordinate': avg_coord,
                            'duration': duration,
                            'nearby_pois': nearby_pois
                        })
                    
                    current_stop = None
        
        # 检查最后一个可能的停止事件
        if current_stop is not None:
            duration = current_stop['timestamps'][-1] - current_stop['timestamps'][0]
            
            if duration >= duration_threshold:
                # 计算停止位置的平均坐标
                avg_lat = sum(c.latitude for c in current_stop['coordinates']) / len(current_stop['coordinates'])
                avg_lon = sum(c.longitude for c in current_stop['coordinates']) / len(current_stop['coordinates'])
                avg_coord = Coordinate(avg_lat, avg_lon)
                
                # 查找附近的POI
                nearby_pois = self.detect_nearby_pois(avg_coord, 100.0)
                
                stop_events.append({
                    'start_index': current_stop['start_index'],
                    'end_index': len(trajectory) - 1,
                    'coordinate': avg_coord,
                    'duration': duration,
                    'nearby_pois': nearby_pois
                })
        
        return stop_events
    
    def detect_junction_crossings(self, trajectory: List[Coordinate], 
                                junction_radius: float = 20.0) -> List[Dict[str, Any]]:
        """
        检测轨迹中的路口穿越事件
        
        参数:
            trajectory: 轨迹坐标列表
            junction_radius: 路口半径 (米)
            
        返回:
            路口穿越事件列表，每个事件是一个字典:
            {
                'index': 穿越点在轨迹中的索引,
                'coordinate': 穿越点坐标,
                'junction': 路口POI对象,
                'connected_roads': 连接的道路列表
            }
        """
        if len(trajectory) < 2:
            return []
        
        # 找出所有类型为"junction"的POI
        junctions = [poi for poi in self.graph.nodes.values() if poi.poi_type == "junction"]
        
        # 如果没有显式标记为junction的POI，则将所有连接多条道路的POI视为路口
        if not junctions:
            junctions = [poi for poi in self.graph.nodes.values() 
                       if len(poi.connected_road_ids) > 2]
        
        junction_crossings = []
        visited_junctions = set()
        
        for i, coord in enumerate(trajectory):
            for junction in junctions:
                junction_id = junction.poi_id
                
                # 如果已经访问过该路口，跳过
                if junction_id in visited_junctions:
                    continue
                
                distance = haversine_distance(coord, junction.coordinate)
                
                if distance <= junction_radius:
                    # 获取连接的道路
                    connected_roads = []
                    for road_id in junction.connected_road_ids:
                        if road_id in self.graph.edges:
                            connected_roads.append(self.graph.edges[road_id])
                    
                    junction_crossings.append({
                        'index': i,
                        'coordinate': coord,
                        'junction': junction,
                        'connected_roads': connected_roads
                    })
                    
                    # 标记为已访问
                    visited_junctions.add(junction_id)
        
        return junction_crossings
    
    def detect_road_changes(self, matched_trajectory: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """
        检测轨迹中的道路变更事件
        
        参数:
            matched_trajectory: 匹配到道路上的轨迹列表
            
        返回:
            道路变更事件列表，每个事件是一个字典:
            {
                'index': 变更点在轨迹中的索引,
                'coordinate': 变更点坐标,
                'from_road': 变更前的道路对象,
                'to_road': 变更后的道路对象
            }
        """
        if not matched_trajectory or len(matched_trajectory) < 2:
            return []
        
        road_changes = []
        current_road_id = None
        
        for i, match in enumerate(matched_trajectory):
            if match is None:
                continue
            
            road_id = match.get('road_id')
            
            if road_id is not None and road_id != current_road_id and current_road_id is not None:
                road_changes.append({
                    'index': i,
                    'coordinate': match.get('matched_coordinate'),
                    'from_road': self.graph.edges.get(current_road_id),
                    'to_road': self.graph.edges.get(road_id)
                })
            
            current_road_id = road_id
        
        return road_changes
    
    def analyze_trajectory(self, trajectory: List[Coordinate], timestamps: Optional[List[float]] = None) -> Dict[str, Any]:
        """
        综合分析轨迹，检测各种事件
        
        参数:
            trajectory: 轨迹坐标列表
            timestamps: 对应的时间戳列表 (秒)，如果为None则不检测与时间相关的事件
            
        返回:
            包含分析结果的字典:
            {
                'matched_trajectory': 匹配到道路上的轨迹,
                'turn_events': 转弯事件列表,
                'junction_crossings': 路口穿越事件列表,
                'road_changes': 道路变更事件列表,
                'stop_events': 停止事件列表 (如果提供了timestamps)
            }
        """
        # 匹配轨迹到道路上
        matched_trajectory = self.match_trajectory(trajectory)
        
        # 检测转弯事件
        turn_events = self.detect_turn_events(trajectory)
        
        # 检测路口穿越事件
        junction_crossings = self.detect_junction_crossings(trajectory)
        
        # 检测道路变更事件
        road_changes = self.detect_road_changes(matched_trajectory)
        
        result = {
            'matched_trajectory': matched_trajectory,
            'turn_events': turn_events,
            'junction_crossings': junction_crossings,
            'road_changes': road_changes
        }
        
        # 如果提供了时间戳，检测停止事件
        if timestamps is not None:
            if len(timestamps) != len(trajectory):
                raise ValueError("轨迹坐标列表和时间戳列表长度不一致")
            
            stop_events = self.detect_stop_events(trajectory, timestamps)
            result['stop_events'] = stop_events
        
        return result
    
    def update_observer(self, observer: Observer) -> Dict[str, Any]:
        """
        更新观察者的感知
        
        参数:
            observer: 观察者对象
            
        返回:
            包含感知结果的字典
        """
        # 匹配观察者位置到道路上
        matched_road = self.match_coordinate(observer.location)
        observer.set_matched_road(matched_road)
        
        # 获取可见的POI和道路
        visible_pois = []
        visible_roads = []
        
        for poi in self.graph.nodes.values():
            if observer.is_poi_visible(poi):
                visible_pois.append(poi)
        
        for road in self.graph.edges.values():
            if observer.is_road_visible(road):
                visible_roads.append(road)
        
        # 更新观察者的感知
        perception_result = observer.update_perception(visible_pois, visible_roads)
        
        return perception_result
    
    def get_navigation_instructions(self, path: List[str]) -> List[Dict[str, Any]]:
        """
        根据路径生成导航指令
        
        参数:
            path: 路径上的节点ID列表
            
        返回:
            导航指令列表，每个指令是一个字典:
            {
                'type': 指令类型 ('start', 'continue', 'turn', 'arrive'),
                'road_name': 道路名称,
                'distance': 距离 (米),
                'direction': 方向 (对于转弯指令),
                'angle': 转弯角度 (度，对于转弯指令),
                'coordinate': 指令位置坐标
            }
        """
        if len(path) < 2:
            return []
        
        # 获取路径上的边
        edges = self.graph.get_path_edges(path)
        
        instructions = []
        current_road_name = None
        current_road_type = None
        accumulated_distance = 0.0
        
        # 添加起点指令
        start_node = self.graph.get_node(path[0])
        start_edge = edges[0]
        
        instructions.append({
            'type': 'start',
            'road_name': start_edge.name or f"{start_edge.road_type} road",
            'road_type': start_edge.road_type,
            'distance': 0.0,
            'coordinate': start_node.coordinate
        })
        
        current_road_name = start_edge.name
        current_road_type = start_edge.road_type
        
        # 处理路径上的每条边
        for i in range(len(edges)):
            edge = edges[i]
            accumulated_distance += edge.length
            
            # 如果道路名称或类型发生变化，添加转弯指令
            if edge.name != current_road_name or edge.road_type != current_road_type:
                # 获取转弯节点
                turn_node_id = path[i]
                turn_node = self.graph.get_node(turn_node_id)
                
                # 计算转弯角度和方向
                if i > 0:
                    prev_edge = edges[i - 1]
                    
                    # 获取前一条边和当前边的方向
                    prev_bearing = bearing(prev_edge.coordinates[-2], prev_edge.coordinates[-1])
                    curr_bearing = bearing(edge.coordinates[0], edge.coordinates[1])
                    
                    # 计算转弯角度
                    angle_diff = abs((curr_bearing - prev_bearing + 180) % 360 - 180)
                    
                    # 判断转弯方向
                    if angle_diff < 10:
                        direction = 'straight'
                    elif angle_diff < 45:
                        direction = 'slight right' if (curr_bearing - prev_bearing + 360) % 360 < 180 else 'slight left'
                    elif angle_diff < 135:
                        direction = 'right' if (curr_bearing - prev_bearing + 360) % 360 < 180 else 'left'
                    else:
                        direction = 'sharp right' if (curr_bearing - prev_bearing + 360) % 360 < 180 else 'sharp left'
                    
                    instructions.append({
                        'type': 'turn',
                        'road_name': edge.name or f"{edge.road_type} road",
                        'road_type': edge.road_type,
                        'distance': accumulated_distance,
                        'direction': direction,
                        'angle': angle_diff,
                        'coordinate': turn_node.coordinate
                    })
                    
                    # 重置累计距离
                    accumulated_distance = 0.0
                
                current_road_name = edge.name
                current_road_type = edge.road_type
            
            # 如果是最后一条边，添加到达指令
            if i == len(edges) - 1:
                end_node = self.graph.get_node(path[-1])
                
                instructions.append({
                    'type': 'arrive',
                    'road_name': current_road_name or f"{current_road_type} road",
                    'road_type': current_road_type,
                    'distance': accumulated_distance,
                    'coordinate': end_node.coordinate
                })
        
        return instructions


"""
坐标模块 - 提供经纬度坐标的封装和操作
"""
from dataclasses import dataclass
from typing import Tuple, Optional
import math


@dataclass
class Coordinate:
    """
    经纬度坐标类，封装了地理坐标的基本操作
    
    属性:
        latitude: 纬度 (度)
        longitude: 经度 (度)
        elevation: 海拔高度 (米)，可选
    """
    latitude: float
    longitude: float
    elevation: Optional[float] = None
    
    def __post_init__(self):
        """验证坐标值是否在有效范围内"""
        if not (-90 <= self.latitude <= 90):
            raise ValueError(f"纬度必须在 -90 到 90 度之间，当前值: {self.latitude}")
        
        if not (-180 <= self.longitude <= 180):
            raise ValueError(f"经度必须在 -180 到 180 度之间，当前值: {self.longitude}")
    
    def to_tuple(self) -> Tuple[float, float]:
        """返回经纬度坐标元组 (longitude, latitude)"""
        return (self.longitude, self.latitude)
    
    def to_tuple_lat_lon(self) -> Tuple[float, float]:
        """返回纬经度坐标元组 (latitude, longitude)"""
        return (self.latitude, self.longitude)
    
    def __str__(self) -> str:
        """返回人类可读的坐标字符串"""
        if self.elevation is not None:
            return f"({self.latitude:.6f}°, {self.longitude:.6f}°, {self.elevation:.1f}m)"
        return f"({self.latitude:.6f}°, {self.longitude:.6f}°)"
    
    def __eq__(self, other) -> bool:
        """判断两个坐标是否相等"""
        if not isinstance(other, Coordinate):
            return False
        
        # 比较经纬度，考虑浮点数精度
        lat_equal = abs(self.latitude - other.latitude) < 1e-10
        lon_equal = abs(self.longitude - other.longitude) < 1e-10
        
        # 如果两个坐标都有海拔信息，则也比较海拔
        if self.elevation is not None and other.elevation is not None:
            elev_equal = abs(self.elevation - other.elevation) < 1e-3
            return lat_equal and lon_equal and elev_equal
        
        # 如果只有一个坐标有海拔信息，则认为不相等
        elif (self.elevation is None) != (other.elevation is None):
            return False
        
        # 如果都没有海拔信息，只比较经纬度
        return lat_equal and lon_equal
    
    def __hash__(self) -> int:
        """计算坐标的哈希值，使其可以用作字典键"""
        # 将经纬度四舍五入到固定精度，以避免浮点数精度问题
        lat_rounded = round(self.latitude, 10)
        lon_rounded = round(self.longitude, 10)
        
        if self.elevation is not None:
            elev_rounded = round(self.elevation, 3)
            return hash((lat_rounded, lon_rounded, elev_rounded))
        
        return hash((lat_rounded, lon_rounded))

    def haversine_distance(self, other: 'Coordinate') -> float:
        """
        计算两个坐标点之间的球面距离(Haversine公式)
        
        参数:
            other: 另一个坐标点
            
        返回:
            两点之间的距离(米)
        """
        # 地球平均半径(km)
        R = 6371.0
        
        # 将经纬度从度转换为弧度
        lat1 = math.radians(self.latitude)
        lon1 = math.radians(self.longitude)
        lat2 = math.radians(other.latitude)
        lon2 = math.radians(other.longitude)
        
        # 计算差值
        dlat = lat2 - lat1
        dlon = lon2 - lon1
        
        # Haversine公式
        a = (math.sin(dlat / 2)**2 + 
             math.cos(lat1) * math.cos(lat2) * 
             math.sin(dlon / 2)**2)
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
        
        # 计算距离(km转m)
        distance = R * c * 1000
        
        return distance


def parse_coordinate(coord_str: str) -> Coordinate:
    """
    从字符串解析坐标
    
    支持的格式:
    - "latitude,longitude"
    - "latitude,longitude,elevation"
    
    参数:
        coord_str: 坐标字符串
        
    返回:
        Coordinate 对象
    """
    parts = coord_str.strip().split(',')
    
    if len(parts) < 2:
        raise ValueError(f"无效的坐标格式: {coord_str}，需要至少包含纬度和经度")
    
    try:
        latitude = float(parts[0])
        longitude = float(parts[1])
        
        if len(parts) > 2:
            elevation = float(parts[2])
            return Coordinate(latitude, longitude, elevation)
        
        return Coordinate(latitude, longitude)
    
    except ValueError as e:
        raise ValueError(f"无法解析坐标值: {coord_str}") from e


"""
距离计算模块 - 提供各种地理距离计算函数
"""
import math
from typing import Tuple, List, Optional

from ..entities.coordinate import Coordinate


def haversine_distance(coord1: Coordinate, coord2: Coordinate) -> float:
    """
    使用Haversine公式计算两个地理坐标之间的距离
    
    参数:
        coord1: 第一个坐标
        coord2: 第二个坐标
        
    返回:
        两点之间的距离 (米)
    """
    # 地球平均半径 (米)
    EARTH_RADIUS = 6371000
    
    # 将经纬度转换为弧度
    lat1 = math.radians(coord1.latitude)
    lon1 = math.radians(coord1.longitude)
    lat2 = math.radians(coord2.latitude)
    lon2 = math.radians(coord2.longitude)
    
    # Haversine公式
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon/2)**2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
    distance = EARTH_RADIUS * c
    
    # 如果两个点都有海拔信息，考虑海拔差异
    if coord1.elevation is not None and coord2.elevation is not None:
        elevation_diff = coord2.elevation - coord1.elevation
        distance = math.sqrt(distance**2 + elevation_diff**2)
    
    return distance


def vincenty_distance(coord1: Coordinate, coord2: Coordinate, iterations: int = 100, epsilon: float = 1e-12) -> float:
    """
    使用Vincenty公式计算两个地理坐标之间的距离 (更精确但计算更复杂)
    
    参数:
        coord1: 第一个坐标
        coord2: 第二个坐标
        iterations: 最大迭代次数
        epsilon: 收敛阈值
        
    返回:
        两点之间的距离 (米)
    """
    # WGS-84椭球体参数
    a = 6378137.0  # 赤道半径 (米)
    f = 1/298.257223563  # 扁率
    b = (1 - f) * a  # 极半径
    
    # 将经纬度转换为弧度
    lat1 = math.radians(coord1.latitude)
    lon1 = math.radians(coord1.longitude)
    lat2 = math.radians(coord2.latitude)
    lon2 = math.radians(coord2.longitude)
    
    # Vincenty公式
    U1 = math.atan((1 - f) * math.tan(lat1))
    U2 = math.atan((1 - f) * math.tan(lat2))
    L = lon2 - lon1
    
    sinU1 = math.sin(U1)
    cosU1 = math.cos(U1)
    sinU2 = math.sin(U2)
    cosU2 = math.cos(U2)
    
    # 初始化lambda
    lmbda = L
    
    for _ in range(iterations):
        sin_lambda = math.sin(lmbda)
        cos_lambda = math.cos(lmbda)
        
        sin_sigma = math.sqrt((cosU2 * sin_lambda) ** 2 + 
                             (cosU1 * sinU2 - sinU1 * cosU2 * cos_lambda) ** 2)
        
        # 如果两点重合
        if sin_sigma == 0:
            return 0.0
        
        cos_sigma = sinU1 * sinU2 + cosU1 * cosU2 * cos_lambda
        sigma = math.atan2(sin_sigma, cos_sigma)
        
        sin_alpha = cosU1 * cosU2 * sin_lambda / sin_sigma
        cos_sq_alpha = 1 - sin_alpha ** 2
        
        # 避免除以零
        if cos_sq_alpha == 0:
            cos_2sigma_m = 0
        else:
            cos_2sigma_m = cos_sigma - 2 * sinU1 * sinU2 / cos_sq_alpha
        
        C = f / 16 * cos_sq_alpha * (4 + f * (4 - 3 * cos_sq_alpha))
        
        lambda_prev = lmbda
        lmbda = L + (1 - C) * f * sin_alpha * (sigma + C * sin_sigma * 
                (cos_2sigma_m + C * cos_sigma * (-1 + 2 * cos_2sigma_m ** 2)))
        
        # 检查是否收敛
        if abs(lmbda - lambda_prev) < epsilon:
            break
    
    # 计算椭球面距离
    u_sq = cos_sq_alpha * (a ** 2 - b ** 2) / b ** 2
    A = 1 + u_sq / 16384 * (4096 + u_sq * (-768 + u_sq * (320 - 175 * u_sq)))
    B = u_sq / 1024 * (256 + u_sq * (-128 + u_sq * (74 - 47 * u_sq)))
    
    delta_sigma = B * sin_sigma * (cos_2sigma_m + B / 4 * (cos_sigma * (-1 + 2 * cos_2sigma_m ** 2) - 
                                  B / 6 * cos_2sigma_m * (-3 + 4 * sin_sigma ** 2) * 
                                  (-3 + 4 * cos_2sigma_m ** 2)))
    
    distance = b * A * (sigma - delta_sigma)
    
    # 如果两个点都有海拔信息，考虑海拔差异
    if coord1.elevation is not None and coord2.elevation is not None:
        elevation_diff = coord2.elevation - coord1.elevation
        distance = math.sqrt(distance**2 + elevation_diff**2)
    
    return distance


def euclidean_distance(coord1: Coordinate, coord2: Coordinate) -> float:
    """
    计算两个坐标的欧几里得距离 (适用于小范围区域的近似计算)
    
    参数:
        coord1: 第一个坐标
        coord2: 第二个坐标
        
    返回:
        两点之间的近似距离 (米)
    """
    # 地球平均半径 (米)
    EARTH_RADIUS = 6371000
    
    # 将经纬度差转换为近似的米数
    # 这是一个简化的计算，仅适用于小范围区域
    lat_diff = math.radians(coord2.latitude - coord1.latitude) * EARTH_RADIUS
    lon_diff = math.radians(coord2.longitude - coord1.longitude) * EARTH_RADIUS * math.cos(math.radians((coord1.latitude + coord2.latitude) / 2))
    
    # 计算平面距离
    distance = math.sqrt(lat_diff**2 + lon_diff**2)
    
    # 如果两个点都有海拔信息，考虑海拔差异
    if coord1.elevation is not None and coord2.elevation is not None:
        elevation_diff = coord2.elevation - coord1.elevation
        distance = math.sqrt(distance**2 + elevation_diff**2)
    
    return distance


def path_length(coordinates: List[Coordinate], method: str = 'haversine') -> float:
    """
    计算路径的总长度
    
    参数:
        coordinates: 路径上的坐标点列表
        method: 距离计算方法 ('haversine', 'vincenty', 'euclidean')
        
    返回:
        路径总长度 (米)
    """
    if len(coordinates) < 2:
        return 0.0
    
    total_length = 0.0
    
    for i in range(len(coordinates) - 1):
        if method == 'vincenty':
            total_length += vincenty_distance(coordinates[i], coordinates[i + 1])
        elif method == 'euclidean':
            total_length += euclidean_distance(coordinates[i], coordinates[i + 1])
        else:  # 默认使用haversine
            total_length += haversine_distance(coordinates[i], coordinates[i + 1])
    
    return total_length


def bearing(coord1: Coordinate, coord2: Coordinate) -> float:
    """
    计算从一个坐标点到另一个坐标点的方位角
    
    参数:
        coord1: 起始坐标
        coord2: 目标坐标
        
    返回:
        方位角 (度，北为0度，顺时针增加)
    """
    # 将经纬度转换为弧度
    lat1 = math.radians(coord1.latitude)
    lon1 = math.radians(coord1.longitude)
    lat2 = math.radians(coord2.latitude)
    lon2 = math.radians(coord2.longitude)
    
    # 计算方位角
    dlon = lon2 - lon1
    y = math.sin(dlon) * math.cos(lat2)
    x = math.cos(lat1) * math.sin(lat2) - math.sin(lat1) * math.cos(lat2) * math.cos(dlon)
    bearing_rad = math.atan2(y, x)
    
    # 转换为度数并确保在0-360范围内
    bearing_deg = math.degrees(bearing_rad)
    bearing_deg = (bearing_deg + 360) % 360
    
    return bearing_deg


def midpoint(coord1: Coordinate, coord2: Coordinate) -> Coordinate:
    """
    计算两个坐标的中点
    
    参数:
        coord1: 第一个坐标
        coord2: 第二个坐标
        
    返回:
        中点坐标
    """
    # 将经纬度转换为弧度
    lat1 = math.radians(coord1.latitude)
    lon1 = math.radians(coord1.longitude)
    lat2 = math.radians(coord2.latitude)
    lon2 = math.radians(coord2.longitude)
    
    # 计算中点
    Bx = math.cos(lat2) * math.cos(lon2 - lon1)
    By = math.cos(lat2) * math.sin(lon2 - lon1)
    
    lat3 = math.atan2(math.sin(lat1) + math.sin(lat2),
                     math.sqrt((math.cos(lat1) + Bx) ** 2 + By ** 2))
    lon3 = lon1 + math.atan2(By, math.cos(lat1) + Bx)
    
    # 转换回度数
    lat3_deg = math.degrees(lat3)
    lon3_deg = math.degrees(lon3)
    
    # 计算海拔中点（如果有）
    elevation = None
    if coord1.elevation is not None and coord2.elevation is not None:
        elevation = (coord1.elevation + coord2.elevation) / 2
    
    return Coordinate(lat3_deg, lon3_deg, elevation)


def destination_point(start: Coordinate, bearing_deg: float, distance: float) -> Coordinate:
    """
    计算从起点出发，沿指定方位角行进指定距离后的终点坐标
    
    参数:
        start: 起始坐标
        bearing_deg: 方位角 (度，北为0度，顺时针增加)
        distance: 距离 (米)
        
    返回:
        终点坐标
    """
    # 地球平均半径 (米)
    EARTH_RADIUS = 6371000
    
    # 将输入转换为弧度
    lat1 = math.radians(start.latitude)
    lon1 = math.radians(start.longitude)
    bearing_rad = math.radians(bearing_deg)
    
    # 计算角距离
    angular_distance = distance / EARTH_RADIUS
    
    # 计算终点坐标
    lat2 = math.asin(math.sin(lat1) * math.cos(angular_distance) + 
                    math.cos(lat1) * math.sin(angular_distance) * math.cos(bearing_rad))
    
    lon2 = lon1 + math.atan2(math.sin(bearing_rad) * math.sin(angular_distance) * math.cos(lat1),
                           math.cos(angular_distance) - math.sin(lat1) * math.sin(lat2))
    
    # 转换回度数
    lat2_deg = math.degrees(lat2)
    lon2_deg = math.degrees(lon2)
    
    # 保持经度在-180到180度之间
    lon2_deg = ((lon2_deg + 180) % 360) - 180
    
    return Coordinate(lat2_deg, lon2_deg, start.elevation)